@@ -1229,10 +1229,16 @@ static void destroy_icon(FvwmWindow *tmp_win)
     XDestroyWindow(dpy, tmp_win->icon_w);
     XDeleteContext(dpy, tmp_win->icon_w, FvwmContext);
   }
-  if((IS_ICON_OURS(tmp_win))&&(tmp_win->icon_pixmap_w != None))
-    XDestroyWindow(dpy, tmp_win->icon_pixmap_w);
-  if(tmp_win->icon_pixmap_w != None)
+  if (tmp_win->icon_pixmap_w != None)
   {
+    if (IS_ICON_OURS(tmp_win))
+    {
+      XDestroyWindow(dpy, tmp_win->icon_pixmap_w);
+    }
+    else
+    {
+      XUnmapWindow(dpy, tmp_win->icon_pixmap_w);
+    }
     XDeleteContext(dpy, tmp_win->icon_pixmap_w, FvwmContext);
   }
   tmp_win->icon_pixmap_w = None;
@@ -1569,6 +1575,14 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   /****** grab keys and buttons ******/
   setup_key_and_button_grabs(tmp_win);
 
+  /****** now we can sefely ungrab the server ******/
+  MyXUngrabServer(dpy);
+
+  /****** inform modules of new window ******/
+  BroadcastConfig(M_ADD_WINDOW,tmp_win);
+  BroadcastName(M_WINDOW_NAME,tmp_win->w,tmp_win->frame,
+		(unsigned long)tmp_win,tmp_win->name);
+
   /****** place the window in the stack ring ******/
   if (!position_new_window_in_stack_ring(tmp_win, SDO_START_LOWERED(sflags)))
   {
@@ -1578,14 +1592,6 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
     XConfigureWindow(dpy, tmp_win->frame, CWSibling|CWStackMode, &xwc);
   }
 
-  /****** now we can sefely ungrab the server ******/
-  MyXUngrabServer(dpy);
-
-  /****** inform modules of new window ******/
-  BroadcastConfig(M_ADD_WINDOW,tmp_win);
-  BroadcastName(M_WINDOW_NAME,tmp_win->w,tmp_win->frame,
-		(unsigned long)tmp_win,tmp_win->name);
-
   /* these are sent and broadcast before res_{class,name} for the benefit
    * of FvwmIconBox which can't handle M_ICON_FILE after M_RES_NAME */
   /****** icon and mini icon ******/
@@ -1950,7 +1956,15 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 	broken_cause = "aspect ratio";
       tmp->hints.flags &= ~PAspect;
       fvwm_msg (WARN, "GetWindowSizeHints",
-		"%s window %#lx has broken aspect ratio: %d/%d - %d/%d\n",
+		"The applicaton window \"%s\"\n"
+		"(window id %#lx)"
+		" has broken aspect ratio: %d/%d - %d/%d\n"
+		"Fvwm is ignoring the aspect ratio.  "
+		"If you are having a problem\n"
+		"with the application, send this message to"
+		" the application owner.\n"
+		"There is no need to notify "
+		"fvwm-workers@fvwm.org.\n",
 		tmp->name, tmp->w, minAspectX, minAspectY, maxAspectX,
 	  	maxAspectY);
     }
@@ -1991,9 +2005,13 @@ void GetWindowSizeHints(FvwmWindow *tmp)
   if (*broken_cause != 0)
   {
     fvwm_msg(WARN, "GetWindowSizeHints",
-	     "%s window %#lx has broken (%s) size hints\n"
-	     "Please send a bug report to the application owner,\n"
-	     "\tyou may use fvwm-workers@fvwm.org as a reference.\n"
+	     "The application window \"%s\"\n(window id %#lx) "
+	     "has broken size hints (%s).\n"
+	     "Fvwm is ignoring those hints.  "
+	     "If you are having a problem\n"
+	     "with the application, send this message to"
+	     " the application owner.\n"
+	     "There is no need to notify fvwm-workers@fvwm.org.\n"
 	     "hint override = %d\n"
 	     "flags = %lx\n"
 	     "min_width = %d, min_height = %d, "
