@@ -30,6 +30,7 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/fvwmsignal.h"
+#include "libs/FShape.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "libs/Colorset.h"
@@ -55,6 +56,7 @@
 #include "style.h"
 #include "focus.h"
 #include "stack.h"
+#include "move_resize.h"
 #ifdef HAVE_STROKE
 #include "stroke.h"
 #endif /* HAVE_STROKE */
@@ -117,18 +119,33 @@ void CMD_WindowShade(F_CMD_ARGS)
   int move_parent_too = False;
   rectangle frame_g;
   rectangle parent_g;
+  rectangle shape_g;
   rectangle diff;
   rectangle pdiff;
+  rectangle sdiff;
   rectangle big_g;
   rectangle small_g;
   Bool do_scroll;
   FvwmWindow *sf;
+  static Window shape_w = None;
 
   if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_SELECT,ButtonRelease))
     return;
   if (tmp_win == NULL || IS_ICONIFIED(tmp_win))
     return;
 
+  if (FShapesSupported && shape_w == None && tmp_win->wShaped)
+  {
+    XSetWindowAttributes attributes;
+    unsigned long valuemask;
+
+    valuemask = CWOverrideRedirect;
+    attributes.override_redirect = True;
+    shape_w = XCreateWindow(
+      dpy, Scr.Root, -32768, -32768, 1, 1, 0, CopyFromParent,
+      (unsigned int) CopyFromParent, CopyFromParent, valuemask, &attributes);
+  }
+
   sf = get_focus_window();
   /* parse arguments */
   toggle = ParseToggleArgument(action, NULL, -1, 0);
@@ -199,6 +216,20 @@ void CMD_WindowShade(F_CMD_ARGS)
       pdiff.y = 0;
       pdiff.width = 0;
       pdiff.height = step;
+      if (client_grav == SouthEastGravity)
+      {
+	shape_g.x = bwl;
+	shape_g.y = bht - ch;
+	sdiff.x = 0;
+	sdiff.y = step;
+      }
+      else
+      {
+	shape_g.x = bwl;
+	shape_g.y = bht;
+	sdiff.x = 0;
+	sdiff.y = 0;
+      }
       diff.x = 0;
       diff.y = HAS_BOTTOM_TITLE(tmp_win) ? -step : 0;
       diff.width = 0;
@@ -230,19 +261,56 @@ void CMD_WindowShade(F_CMD_ARGS)
 	parent_g.y += pdiff.y;
 	parent_g.width += pdiff.width;
 	parent_g.height += pdiff.height;
+	shape_g.x += sdiff.x;
+	shape_g.y += sdiff.y;
 	frame_g.x += diff.x;
 	frame_g.y += diff.y;
 	frame_g.width += diff.width;
 	frame_g.height += diff.height;
+	if (FShapesSupported && tmp_win->wShaped && shape_w)
+	{
+	  FShapeCombineShape(
+	    dpy, shape_w, FShapeBounding, shape_g.x, shape_g.y, tmp_win->w,
+	    FShapeBounding, FShapeSet);
+	  if (tmp_win->title_w)
+	  {
+	    XRectangle rect;
+
+	    /* windows w/ titles */
+	    rect.x = bwl;
+	    rect.y = (HAS_BOTTOM_TITLE(tmp_win)) ? frame_g.height - bhb : 0;
+	    rect.width = frame_g.width - bw;
+	    rect.height = tmp_win->title_g.height;
+	    FShapeCombineRectangles(
+	      dpy, shape_w, FShapeBounding, 0, 0, &rect, 1, FShapeUnion,
+	      Unsorted);
+	  }
+	}
 	if (move_parent_too)
+	{
+	  if (FShapesSupported && tmp_win->wShaped && shape_w)
+	  {
+	    FShapeCombineShape(
+	      dpy, tmp_win->frame, FShapeBounding, 0, 0, shape_w,
+	      FShapeBounding, FShapeUnion);
+	  }
 	  XMoveResizeWindow(
 	    dpy, tmp_win->Parent, parent_g.x, parent_g.y, parent_g.width,
 	    parent_g.height);
+	}
 	else
+	{
 	  XResizeWindow(dpy, tmp_win->Parent, parent_g.width, parent_g.height);
+	}
 	XMoveResizeWindow(
 	  dpy, tmp_win->frame, frame_g.x, frame_g.y, frame_g.width,
 	  frame_g.height);
+	if (FShapesSupported && tmp_win->wShaped && shape_w)
+	{
+	  FShapeCombineShape(
+	    dpy, tmp_win->frame, FShapeBounding, 0, 0, shape_w, FShapeBounding,
+	    FShapeSet);
+	}
         FlushAllMessageQueues();
         XSync(dpy, 0);
       }
@@ -300,6 +368,14 @@ void CMD_WindowShade(F_CMD_ARGS)
     /* shade window */
     SET_SHADED(tmp_win, 1);
     get_shaded_geometry(tmp_win, &small_g, &big_g);
+    if (small_g.width < 1)
+    {
+      small_g.width = 1;
+    }
+    if (small_g.height < 1)
+    {
+      small_g.height = 1;
+    }
     if (tmp_win->shade_anim_steps != 0)
     {
       parent_g.x = bwl;
@@ -310,6 +386,9 @@ void CMD_WindowShade(F_CMD_ARGS)
       pdiff.y = 0;
       pdiff.width = 0;
       pdiff.height = -step;
+      shape_g = parent_g;
+      sdiff.x = 0;
+      sdiff.y = (client_grav == SouthEastGravity) ? -step : 0;
       diff.x = 0;
       diff.y = HAS_BOTTOM_TITLE(tmp_win) ? step : 0;
       diff.width = 0;
@@ -321,12 +400,39 @@ void CMD_WindowShade(F_CMD_ARGS)
 	parent_g.y += pdiff.y;
 	parent_g.width += pdiff.width;
 	parent_g.height += pdiff.height;
+	shape_g.x += sdiff.x;
+	shape_g.y += sdiff.y;
 	frame_g.x += diff.x;
 	frame_g.y += diff.y;
 	frame_g.width += diff.width;
 	frame_g.height += diff.height;
+	if (FShapesSupported && tmp_win->wShaped && shape_w)
+	{
+	  FShapeCombineShape(
+	    dpy, shape_w, FShapeBounding, shape_g.x, shape_g.y, tmp_win->w,
+	    FShapeBounding, FShapeSet);
+	  if (tmp_win->title_w)
+	  {
+	    XRectangle rect;
+
+	    /* windows w/ titles */
+	    rect.x = bwl;
+	    rect.y = (HAS_BOTTOM_TITLE(tmp_win)) ? frame_g.height - bhb : 0;
+	    rect.width = big_g.width - bw;
+	    rect.height = tmp_win->title_g.height;
+	    FShapeCombineRectangles(
+	      dpy, shape_w, FShapeBounding, 0, 0, &rect, 1, FShapeUnion,
+	      Unsorted);
+	  }
+	}
 	if (move_parent_too)
 	{
+	  if (FShapesSupported && tmp_win->wShaped && shape_w)
+	  {
+	    FShapeCombineShape(
+	      dpy, tmp_win->frame, FShapeBounding, 0, 0, shape_w,
+	      FShapeBounding, FShapeUnion);
+	  }
 	  XMoveResizeWindow(
 	    dpy, tmp_win->frame, frame_g.x, frame_g.y, frame_g.width,
 	    frame_g.height);
@@ -341,6 +447,12 @@ void CMD_WindowShade(F_CMD_ARGS)
 	    dpy, tmp_win->frame, frame_g.x, frame_g.y, frame_g.width,
 	    frame_g.height);
 	}
+	if (FShapesSupported && tmp_win->wShaped && shape_w)
+	{
+	  FShapeCombineShape(
+	    dpy, tmp_win->frame, FShapeBounding, 0, 0, shape_w, FShapeBounding,
+	    FShapeSet);
+	}
         FlushAllMessageQueues();
         XSync(dpy, 0);
       }
@@ -670,6 +782,8 @@ void CMD_Exec(F_CMD_ARGS)
   /* Not everyone has vfork! */
   if (!(fork())) /* child process */
   {
+    /* close stdin so the exec'd process knows its not interactive */
+    close(0);
     if (execl(exec_shell_name, exec_shell_name, "-c", cmd, NULL)==-1)
     {
       fvwm_msg(ERR,"exec_function","execl failed (%s)",strerror(errno));
@@ -951,7 +1065,7 @@ void CMD_HilightColor(F_CMD_ARGS)
   if (Scr.cur_decor && Scr.cur_decor != &Scr.DefaultDecor)
   {
     fvwm_msg(
-      ERR, "SetHiColor",
+      ERR, "CMD_HilightColor",
       "Decors do not support the HilightColor command anymore."
       " Please use 'Style <stylename> HilightFore <forecolor>' and"
       " 'Style <stylename> HilightBack <backcolor>' instead."
@@ -982,7 +1096,7 @@ void CMD_HilightColorset(F_CMD_ARGS)
   if (Scr.cur_decor && Scr.cur_decor != &Scr.DefaultDecor)
   {
     fvwm_msg(
-      ERR, "SetHiColorset",
+      ERR, "CMD_HilightColorset",
       "Decors do not support the HilightColorset command anymore."
       " Please use 'Style <stylename> HilightColorset <colorset>' instead."
       " Sorry for the inconvenience.");
@@ -1080,8 +1194,6 @@ void ApplyDefaultFontAndColors(void)
 {
   XGCValues gcv;
   unsigned long gcm;
-  int wid;
-  int hei;
   int cset = Scr.DefaultColorset;
 
   /* make GC's */
@@ -1123,31 +1235,7 @@ void ApplyDefaultFontAndColors(void)
   /* update the geometry window for move/resize */
   if(Scr.SizeWindow != None)
   {
-    Scr.SizeStringWidth =
-      XTextWidth(Scr.DefaultFont.font, " +8888 x +8888 ", 15);
-    wid = Scr.SizeStringWidth + 2 * SIZE_HINDENT;
-    hei = Scr.DefaultFont.height + 2 * SIZE_VINDENT;
-    if (cset >= 0)
-    {
-      SetWindowBackground(
-	dpy, Scr.SizeWindow, wid, hei, &Colorset[cset], Pdepth, Scr.StdGC,
-	False);
-    }
-    else
-    {
-      XSetWindowBackground(dpy, Scr.SizeWindow, Scr.StdBack);
-    }
-
-    if(Scr.gs.EmulateMWM)
-    {
-      XMoveResizeWindow(
-	dpy, Scr.SizeWindow, (Scr.MyDisplayWidth - wid) / 2,
-	(Scr.MyDisplayHeight - hei) / 2, wid, hei);
-    }
-    else
-    {
-      XMoveResizeWindow(dpy, Scr.SizeWindow, 0, 0, wid, hei);
-    }
+    resize_geometry_window();
   }
 
   UpdateAllMenuStyles();
@@ -1158,6 +1246,21 @@ void CMD_Colorset(F_CMD_ARGS)
   int n = -1, ret;
   char *token;
 
+  /* migo (19/May/2002): this block also fixes a crash with new configs */
+  char *ptr;
+  for (ptr = action; *ptr; ptr++)
+  {
+    if (*ptr != ' ' && !isxdigit(*ptr))
+    {
+      /* suppose it is a new Colorset command and pass it to FvwmTheme */
+      char *old_action = action;
+      action = CatString2("FvwmTheme Colorset ", action);
+      CMD_SendToModule(F_PASS_ARGS);
+      action = old_action;
+      return;
+    }
+  }
+
   token = PeekToken(action, NULL);
   if (token == NULL)
     return;
@@ -1166,7 +1269,9 @@ void CMD_Colorset(F_CMD_ARGS)
   if ((ret == 0) || (n < 0))
     return;
 
-  LoadColorset(action);
+  /* add a check to avoid crash in intentionally-incorect-config */
+  if (LoadColorset(action) < 0)
+    return;
   BroadcastColorset(n);
 
   if (n == Scr.DefaultColorset)
@@ -1450,6 +1555,7 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
 
       vc->num = num_coords;
       vc->line_style = 0;
+      vc->use_fgbg = 0;
       vc->x = (int *) safemalloc (sizeof (int) * num_coords);
       vc->y = (int *) safemalloc (sizeof (int) * num_coords);
 
@@ -1472,10 +1578,14 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
 	  vc->y = NULL;
 	  return False;
 	}
-	if (line_style)
+	if (line_style % 2)
 	{
 	  vc->line_style |= (1 << i);
 	}
+	if (line_style >= 2)
+	{
+	  vc->use_fgbg |= (1 << i);
+	}
 	s += offset;
       }
       memset(&df->style, 0, sizeof(df->style));
@@ -1769,7 +1879,7 @@ static char *ReadTitleButton(
     {
       for (i = 0; i < BS_MaxButtonState; ++i)
       {
-	ReadDecorFace(spec, &TB_STATE(*tb)[i], BS_All, !i);
+	ReadDecorFace(spec, &TB_STATE(*tb)[i], button, !i);
       }
     }
     else
@@ -2068,9 +2178,7 @@ static void DestroyFvwmDecor(FvwmDecor *decor)
   DestroyAllButtons(decor);
   for (i = 0; i < BS_MaxButtonState; ++i)
   {
-    int j = 0;
-    for (; j < BS_MaxButtonState; ++j)
-      FreeDecorFace(dpy, &TB_STATE(decor->titlebar)[i]);
+    FreeDecorFace(dpy, &TB_STATE(decor->titlebar)[i]);
   }
   FreeDecorFace(dpy, &decor->BorderStyle.active);
   FreeDecorFace(dpy, &decor->BorderStyle.inactive);
@@ -2495,6 +2603,8 @@ void CMD_SetEnv(F_CMD_ARGS)
   action = GetNextToken(action,&szValue);
   if (!szValue)
   {
+    /* no value, treat as unset */
+    CMD_UnsetEnv(F_PASS_ARGS);
     free(szVar);
     return;
   }
@@ -3053,7 +3163,7 @@ void CMD_StrokeFunc(F_CMD_ARGS)
   int abort = 0;
   int modifiers = eventp->xbutton.state;
   int start_event_type = eventp->type;
-  char sequence[MAX_SEQUENCE+1];
+  char sequence[STROKE_MAX_SEQUENCE + 1];
   char *stroke_action;
   char *opt = NULL;
   Bool finish_on_release = True;
@@ -3166,6 +3276,10 @@ void CMD_StrokeFunc(F_CMD_ARGS)
     switch (eventp->type)
     {
     case MotionNotify:
+      if (eventp->xmotion.same_screen == False)
+      {
+	continue;
+      }
       if (eventp->xany.window != Scr.Root)
       {
 	XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild, &tmpx, &tmpy,
@@ -3255,7 +3369,7 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 
   if (echo_sequence)
   {
-    char num_seq[MAX_SEQUENCE+1];
+    char num_seq[STROKE_MAX_SEQUENCE + 1];
 
     for(i=0;sequence[i] != '\0';i++)
     {
