@@ -44,8 +44,14 @@
 #include "borders.h"
 #include "virtual.h"
 #include "stack.h"
+#include "colormaps.h"
 
 static Bool lastFocusType;
+/* Last window which Fvwm gave the focus to NOT the window that really has the
+ * focus */
+static FvwmWindow *ScreenFocus = NULL;
+/* Window which had focus before the pointer moved to a different screen. */
+static FvwmWindow *LastScreenFocus = NULL;
 
 Bool do_accept_input_focus(FvwmWindow *tmp_win)
 {
@@ -61,14 +67,18 @@ Bool do_accept_input_focus(FvwmWindow *tmp_win)
 static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
 {
   extern Time lastTimestamp;
+  FvwmWindow *sf;
 
+  if (Fw && WM_TAKES_FOCUS(Fw))
+  {
+    send_clientmessage(dpy, w, _XA_WM_TAKE_FOCUS, lastTimestamp);
+  }
   if (Fw && HAS_NEVER_FOCUS(Fw))
   {
     if (WM_TAKES_FOCUS(Fw))
     {
       /* give it a chance to take the focus itself */
-      send_clientmessage(dpy, w, _XA_WM_TAKE_FOCUS, lastTimestamp);
-      XSync(dpy,0);
+      XSync(dpy, 0);
     }
     else
     {
@@ -77,20 +87,17 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
     }
     return;
   }
-
   if (Fw && !IS_LENIENT(Fw) &&
       Fw->wmhints && (Fw->wmhints->flags & InputHint) && !Fw->wmhints->input &&
-      Scr.Focus && Scr.Focus->Desk == Scr.CurrentDesk)
+      (sf = get_focus_window()) && sf->Desk == Scr.CurrentDesk)
   {
-    /* window doesn't want focus */
     return;
   }
-
   /* ClickToFocus focus queue manipulation - only performed for
    * Focus-by-mouse type focus events */
   /* Watch out: Fw may not be on the windowlist and the windowlist may be
    * empty */
-  if (Fw && Fw != Scr.Focus && Fw != &Scr.FvwmRoot)
+  if (Fw && Fw != get_focus_window() && Fw != &Scr.FvwmRoot)
   {
     if (FocusByMouse) /* pluck window from list and deposit at top */
     {
@@ -140,14 +147,14 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
   if (!Fw && !Scr.flags.is_pointer_on_this_screen)
   {
     focus_grab_buttons(Scr.Ungrabbed, False);
-    Scr.Focus = NULL;
+    set_focus_window(NULL);
     /* DV (25-Nov-2000): Don't give the Scr.NoFocusWin the focus here. This
      * would steal the focus from the other screen's root window again. */
     /* FOCUS_SET(Scr.NoFocusWin); */
     return;
   }
 
-  if(Fw != NULL && !NoWarp)
+  if (Fw && !NoWarp)
   {
     if (IS_ICONIFIED(Fw))
     {
@@ -178,8 +185,7 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
   */
   if (Scr.Ungrabbed != Fw)
   {
-    /* need to grab all buttons for window that we are about to
-     * unfocus */
+    /* need to grab all buttons for window that we are about to unfocus */
     focus_grab_buttons(Scr.Ungrabbed, False);
   }
   /* if we do click to focus, remove the grab on mouse events that
@@ -190,22 +196,27 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
   }
   /* RBW - allow focus to go to a NoIconTitle icon window so
    * auto-raise will work on it... */
-  if((Fw)&&(IS_ICONIFIED(Fw)))
+  if (Fw && IS_ICONIFIED(Fw))
   {
+    Bool is_window_selected = False;
+
     if (Fw->icon_w)
     {
       w = Fw->icon_w;
+      is_window_selected = True;
     }
-    else if (Fw->icon_pixmap_w)
+    if ((!is_window_selected || WAS_ICON_HINT_PROVIDED(Fw)) &&
+	Fw->icon_pixmap_w)
     {
       w = Fw->icon_pixmap_w;
     }
   }
 
-  if(Fw && IS_LENIENT(Fw))
+  if (Fw && IS_LENIENT(Fw))
   {
     FOCUS_SET(w);
-    Scr.Focus = Fw;
+    set_focus_window(Fw);
+    SET_FOCUS_CHANGE_BROADCAST_PENDING(Fw, 1);
     Scr.UnknownWinFocused = None;
   }
   else if (!Fw || !(Fw->wmhints) || !(Fw->wmhints->flags & InputHint) ||
@@ -213,10 +224,14 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
   {
     /* Window will accept input focus */
     FOCUS_SET(w);
-    Scr.Focus = Fw;
+    set_focus_window(Fw);
+    if (Fw)
+    {
+      SET_FOCUS_CHANGE_BROADCAST_PENDING(Fw, 1);
+    }
     Scr.UnknownWinFocused = None;
   }
-  else if ((Scr.Focus)&&(Scr.Focus->Desk == Scr.CurrentDesk))
+  else if ((sf = get_focus_window()) && sf->Desk == Scr.CurrentDesk)
   {
     /* Window doesn't want focus. Leave focus alone */
     /* FOCUS_SET(Scr.Hilite->w);*/
@@ -224,33 +239,61 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
   else
   {
     FOCUS_SET(Scr.NoFocusWin);
-    Scr.Focus = NULL;
+    set_focus_window(NULL);
   }
-
-  if (Fw && WM_TAKES_FOCUS(Fw))
-  {
-    send_clientmessage(dpy, w, _XA_WM_TAKE_FOCUS, lastTimestamp);
-  }
-
   XSync(dpy,0);
+
+  return;
 }
 
 static void MoveFocus(
   Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp, Bool do_force)
 {
-  FvwmWindow *ffw_old = Scr.Focus;
+  FvwmWindow *ffw_old = get_focus_window();
   Bool accepts_input_focus = do_accept_input_focus(Fw);
+#if 0
+  FvwmWindow *ffw_new;
+#endif
 
-  if (!do_force && Fw == Scr.Focus)
+  if (!do_force && Fw == ffw_old)
   {
+    if (ffw_old)
+    {
+#if 0
+      /*
+          RBW - 2001/08/17 - For a MouseFocusClickRaises win, we must not drop
+          the grab, since the (potential) click hasn't happened yet.
+      */
+      if ((!HAS_SLOPPY_FOCUS(ffw_old) && !HAS_MOUSE_FOCUS(ffw_old)) ||
+	  !DO_RAISE_MOUSE_FOCUS_CLICK(ffw_old))
+      {
+        focus_grab_buttons(ffw_old, True);
+      }
+#else
+      focus_grab_buttons(ffw_old, True);
+#endif
+    }
     return;
   }
   DoSetFocus(w, Fw, FocusByMouse, NoWarp);
-  if (Scr.Focus != ffw_old)
+  if (get_focus_window() != ffw_old)
   {
     if (accepts_input_focus)
     {
-      focus_grab_buttons(Scr.Focus, True);
+#if 0
+      /*  RBW - Ibid.  */
+      ffw_new = get_focus_window();
+      if (ffw_new)
+      {
+	if ((!HAS_SLOPPY_FOCUS(ffw_new) && !HAS_MOUSE_FOCUS(ffw_new)) ||
+	    !DO_RAISE_MOUSE_FOCUS_CLICK(ffw_new))
+	{
+	  focus_grab_buttons(ffw_new, True);
+	}
+      }
+#else
+      focus_grab_buttons(get_focus_window(), True);
+#endif
     }
     focus_grab_buttons(ffw_old, False);
   }
@@ -261,6 +304,11 @@ void SetFocusWindow(FvwmWindow *Fw, Bool FocusByMouse)
   MoveFocus(Fw->w, Fw, FocusByMouse, False, False);
 }
 
+void ReturnFocusWindow(FvwmWindow *Fw, Bool FocusByMouse)
+{
+  MoveFocus(Fw->w, Fw, FocusByMouse, True, False);
+}
+
 void DeleteFocus(Bool FocusByMouse)
 {
   MoveFocus(Scr.NoFocusWin, NULL, FocusByMouse, False, False);
@@ -273,50 +321,61 @@ void ForceDeleteFocus(Bool FocusByMouse)
 
 /* When a window is unmapped (or destroyed) this function takes care of
  * adjusting the focus window appropriately. */
-void restore_focus_after_unmap(FvwmWindow *tmp_win)
+void restore_focus_after_unmap(
+  FvwmWindow *tmp_win, Bool do_skip_marked_transients)
 {
   extern FvwmWindow *colormap_win;
   FvwmWindow *t = NULL;
   FvwmWindow *set_focus_to = NULL;
 
-  if (tmp_win != Scr.Focus)
-    set_focus_to = tmp_win;
-  if (!set_focus_to &&
-      tmp_win->transientfor != None && tmp_win->transientfor != Scr.Root)
+  if (tmp_win == get_focus_window())
   {
-    for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+    if (tmp_win->transientfor != None && tmp_win->transientfor != Scr.Root)
     {
-      if (t->w == tmp_win->transientfor)
+      for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
       {
-	set_focus_to = t;
-	break;
+	if (t->w == tmp_win->transientfor &&
+	    t->Desk == tmp_win->Desk &&
+	    (!do_skip_marked_transients || !IS_IN_TRANSIENT_SUBTREE(t)))
+	{
+	  set_focus_to = t;
+	  break;
+	}
       }
     }
-  }
-  if (!set_focus_to && HAS_CLICK_FOCUS(tmp_win))
-  {
-    for (t = tmp_win->next; t != NULL; t = t->next)
+    if (!set_focus_to &&
+	(HAS_CLICK_FOCUS(tmp_win) || HAS_SLOPPY_FOCUS(tmp_win)))
     {
-      if (t->Desk == tmp_win->Desk)
+      for (t = tmp_win->next; t != NULL; t = t->next)
       {
-	/* If it is on a different desk we have to look for another window */
-	set_focus_to = t;
-	break;
+	if (t->Desk == tmp_win->Desk &&
+	    !DO_SKIP_CIRCULATE(t) &&
+	    !(DO_SKIP_ICON_CIRCULATE(t) && IS_ICONIFIED(t)) &&
+	    (!do_skip_marked_transients || !IS_IN_TRANSIENT_SUBTREE(t)))
+	{
+	  /* If it is on a different desk we have to look for another window */
+	  set_focus_to = t;
+	  break;
+	}
       }
     }
+    if (set_focus_to && set_focus_to != tmp_win &&
+	set_focus_to->Desk == tmp_win->Desk)
+    {
+      /* Don't transfer focus to windows on other desks */
+      SetFocusWindow(set_focus_to, 1);
+    }
+    if (tmp_win == get_focus_window())
+    {
+      DeleteFocus(1);
+    }
   }
-  if (set_focus_to && set_focus_to != tmp_win &&
-      set_focus_to->Desk == tmp_win->Desk)
-  {
-    /* Don't transfer focus to windows on other desks */
-    SetFocusWindow(set_focus_to, 1);
-  }
-  if(Scr.Focus == tmp_win)
-    DeleteFocus(1);
   if (tmp_win == Scr.pushed_window)
     Scr.pushed_window = NULL;
   if (tmp_win == colormap_win)
-    colormap_win = NULL;
+  {
+    InstallWindowColormaps(set_focus_to);
+  }
 
   return;
 }
@@ -332,6 +391,7 @@ void SetPointerEventPosition(XEvent *eventp, int x, int y)
   case MotionNotify:
     eventp->xbutton.x_root = x;
     eventp->xbutton.y_root = y;
+    eventp->xmotion.same_screen = True;
     break;
   default:
     break;
@@ -349,7 +409,7 @@ void FocusOn(FvwmWindow *t, Bool FocusByMouse, char *action)
 {
   int dx,dy;
   int cx,cy;
-  Bool NoWarp;
+  Bool do_not_warp;
 
   if (t == NULL || HAS_NEVER_FOCUS(t))
   {
@@ -362,14 +422,14 @@ void FocusOn(FvwmWindow *t, Bool FocusByMouse, char *action)
     return;
   }
 
-  if (!(NoWarp = StrEquals(PeekToken(action, NULL), "NoWarp")))
+  if (!(do_not_warp = StrEquals(PeekToken(action, NULL), "NoWarp")))
   {
-    if(t->Desk != Scr.CurrentDesk)
+    if (t->Desk != Scr.CurrentDesk)
     {
       goto_desk(t->Desk);
     }
 
-    if(IS_ICONIFIED(t))
+    if (IS_ICONIFIED(t))
     {
       cx = t->icon_xl_loc + t->icon_g.width/2;
       cy = t->icon_g.y + t->icon_p_height + ICON_HEIGHT(t) / 2;
@@ -379,19 +439,17 @@ void FocusOn(FvwmWindow *t, Bool FocusByMouse, char *action)
       cx = t->frame_g.x + t->frame_g.width/2;
       cy = t->frame_g.y + t->frame_g.height/2;
     }
-
     dx = (cx + Scr.Vx)/Scr.MyDisplayWidth*Scr.MyDisplayWidth;
     dy = (cy +Scr.Vy)/Scr.MyDisplayHeight*Scr.MyDisplayHeight;
-
     MoveViewport(dx,dy,True);
 
     /* If the window is still not visible, make it visible! */
-    if(((t->frame_g.x + t->frame_g.height)< 0)||
-       (t->frame_g.y + t->frame_g.width < 0)||
-       (t->frame_g.x >Scr.MyDisplayWidth)||(t->frame_g.y>Scr.MyDisplayHeight))
+    if (((t->frame_g.x + t->frame_g.height)< 0)||
+	(t->frame_g.y + t->frame_g.width < 0)||
+	(t->frame_g.x >Scr.MyDisplayWidth)||(t->frame_g.y>Scr.MyDisplayHeight))
     {
       SetupFrame(t, 0, 0, t->frame_g.width, t->frame_g.height, False);
-      if(HAS_MOUSE_FOCUS(t) || HAS_SLOPPY_FOCUS(t))
+      if (HAS_MOUSE_FOCUS(t) || HAS_SLOPPY_FOCUS(t))
       {
 	XWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, 2,2);
       }
@@ -399,7 +457,12 @@ void FocusOn(FvwmWindow *t, Bool FocusByMouse, char *action)
   }
 
   UngrabEm(GRAB_NORMAL);
-  MoveFocus(t->w, t, FocusByMouse, NoWarp, 0);
+  if (t->Desk == Scr.CurrentDesk)
+  {
+    MoveFocus(t->w, t, FocusByMouse, do_not_warp, 0);
+  }
+
+  return;
 }
 
 /**************************************************************************
@@ -434,30 +497,37 @@ static void warp_to_fvwm_window(
   }
 
   dx = (cx + Scr.Vx) / Scr.MyDisplayWidth * Scr.MyDisplayWidth;
-  dy = (cy +Scr.Vy) / Scr.MyDisplayHeight * Scr.MyDisplayHeight;
+  dy = (cy + Scr.Vy) / Scr.MyDisplayHeight * Scr.MyDisplayHeight;
 
   MoveViewport(dx,dy,True);
 
   if(IS_ICONIFIED(t))
   {
-    x = t->icon_xl_loc + t->icon_g.width/2;
+    x = t->icon_xl_loc + t->icon_g.width / 2;
     y = t->icon_g.y + t->icon_p_height + ICON_HEIGHT(t) / 2;
   }
   else
   {
-    if (x_unit != Scr.MyDisplayWidth)
+    if (x_unit != Scr.MyDisplayWidth && warp_x >= 0)
       x = t->frame_g.x + warp_x;
-    else
+    else if (x_unit != Scr.MyDisplayWidth)
+      x = t->frame_g.x + t->frame_g.width + warp_x;
+    else if (warp_x >= 0)
       x = t->frame_g.x + (t->frame_g.width - 1) * warp_x / 100;
-    if (y_unit != Scr.MyDisplayHeight)
-      y = t->frame_g.y + warp_y;
     else
+      x = t->frame_g.x + (t->frame_g.width - 1) * (100 + warp_x) / 100;
+
+    if (y_unit != Scr.MyDisplayHeight && warp_y >= 0)
+      y = t->frame_g.y + warp_y;
+    else if (y_unit != Scr.MyDisplayHeight)
+      y = t->frame_g.y + t->frame_g.height + warp_y;
+    else if (warp_y >= 0)
       y = t->frame_g.y + (t->frame_g.height - 1) * warp_y / 100;
+    else
+      y = t->frame_g.y + (t->frame_g.height - 1) * (100 + warp_y) / 100;
   }
-  if (warp_x >= 0 && warp_y >= 0) {
-    XWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, x, y);
-    SetPointerEventPosition(eventp, x, y);
-  }
+  XWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, x, y);
+  SetPointerEventPosition(eventp, x, y);
   RaiseWindow(t);
 
   /* If the window is still not visible, make it visible! */
@@ -473,7 +543,7 @@ static void warp_to_fvwm_window(
 }
 
 
-void flip_focus_func(F_CMD_ARGS)
+void CMD_FlipFocus(F_CMD_ARGS)
 {
   if (DeferExecution(eventp,&w,&tmp_win,&context,CRS_SELECT,ButtonRelease))
     return;
@@ -482,7 +552,7 @@ void flip_focus_func(F_CMD_ARGS)
   FocusOn(tmp_win, TRUE, action);
 }
 
-void focus_func(F_CMD_ARGS)
+void CMD_Focus(F_CMD_ARGS)
 {
   if (DeferExecution(eventp,&w,&tmp_win,&context,CRS_SELECT,ButtonRelease))
     return;
@@ -490,7 +560,7 @@ void focus_func(F_CMD_ARGS)
   FocusOn(tmp_win, FALSE, action);
 }
 
-void warp_func(F_CMD_ARGS)
+void CMD_WarpToWindow(F_CMD_ARGS)
 {
   int val1_unit, val2_unit, n;
   int val1, val2;
@@ -523,14 +593,17 @@ void warp_func(F_CMD_ARGS)
 	return;
       }
       if (val1_unit != Scr.MyDisplayWidth)
-	x = wx + val1;
+	x = val1;
       else
-	x = wx + (ww - 1) * val1 / 100;
-
+	x = (ww - 1) * val1 / 100;
       if (val2_unit != Scr.MyDisplayHeight)
-	y = wy + val2;
+	y = val2;
       else
-	y = wy + (wh - 1) * val2 / 100;
+	y = (wh - 1) * val2 / 100;
+      if (x < 0)
+	x += ww;
+      if (y < 0)
+	y += wh;
     }
     XWarpPointer(dpy, None, w, 0, 0, 0, 0, x, y);
   }
@@ -545,42 +618,75 @@ void focus_grab_buttons(FvwmWindow *tmp_win, Bool is_focused)
 {
   int i;
   Bool accepts_input_focus;
-  Bool do_grab_all = False;
+  Bool do_grab_window = False;
   unsigned char grab_buttons = Scr.buttons2grab;
 
-  if (!tmp_win)
+  if (!tmp_win || IS_SCHEDULED_FOR_DESTROY(tmp_win))
   {
+    /* It's pointless to grab buttons on dying windows */
     return;
   }
   accepts_input_focus = do_accept_input_focus(tmp_win);
-
-  if ((HAS_SLOPPY_FOCUS(tmp_win) || HAS_MOUSE_FOCUS(tmp_win) ||
-       HAS_NEVER_FOCUS(tmp_win)) &&
-      DO_RAISE_MOUSE_FOCUS_CLICK(tmp_win) &&
-      (!is_focused || !is_on_top_of_layer(tmp_win)))
+  if (HAS_SLOPPY_FOCUS(tmp_win) || HAS_MOUSE_FOCUS(tmp_win) ||
+       HAS_NEVER_FOCUS(tmp_win))
   {
-    grab_buttons = ((1 << NUMBER_OF_MOUSE_BUTTONS) - 1);
-    do_grab_all = True;
+    if (DO_RAISE_MOUSE_FOCUS_CLICK(tmp_win) &&
+        (!is_focused || !is_on_top_of_layer(tmp_win)))
+    {
+      grab_buttons = ((1 << NUMBER_OF_MOUSE_BUTTONS) - 1);
+      do_grab_window = True;
+    }
   }
-  else if (HAS_CLICK_FOCUS(tmp_win) &&
-	   (!is_focused || !is_on_top_of_layer(tmp_win)) &&
-	   (!DO_NOT_RAISE_CLICK_FOCUS_CLICK(tmp_win) || accepts_input_focus))
+  else if (HAS_CLICK_FOCUS(tmp_win))
   {
-    grab_buttons = ((1 << NUMBER_OF_MOUSE_BUTTONS) - 1);
-    do_grab_all = True;
+    if (is_focused && !is_on_top_of_layer(tmp_win) &&
+        DO_NOT_RAISE_CLICK_FOCUS_CLICK(tmp_win) && accepts_input_focus)
+    {
+      /* don't grab */
+    }
+    else if ((!is_focused || !is_on_top_of_layer(tmp_win)) &&
+             (!DO_NOT_RAISE_CLICK_FOCUS_CLICK(tmp_win) || accepts_input_focus))
+    {
+      grab_buttons = ((1 << NUMBER_OF_MOUSE_BUTTONS) - 1);
+      do_grab_window = True;
+    }
   }
 
+#if 0
+  /*
+      RBW - If we've come here to grab and all buttons are already grabbed,
+      or to ungrab and none is grabbed, then we've nothing to do.
+  */
+  if ((!is_focused && grab_buttons == tmp_win->grabbed_buttons) ||
+     (is_focused && ((tmp_win->grabbed_buttons & grab_buttons) == 0)))
+  {
+    return;
+  }
+#else
   if (grab_buttons != tmp_win->grabbed_buttons)
+#endif
   {
     Bool do_grab;
 
-    MyXGrabServer (dpy);
-    Scr.Ungrabbed = (do_grab_all) ? NULL : tmp_win;
+    MyXGrabServer(dpy);
+    Scr.Ungrabbed = (do_grab_window) ? NULL : tmp_win;
     for (i = 0; i < NUMBER_OF_MOUSE_BUTTONS; i++)
     {
+#if 0
+      /*  RBW - Set flag for grab or ungrab according to how we were called. */
+      if (!is_focused||1)
+      {
+        do_grab = !!(grab_buttons & (1 << i));
+      }
+      else
+      {
+        do_grab = !(grab_buttons & (1 << i));
+      }
+#else
       if ((grab_buttons & (1 << i)) == (tmp_win->grabbed_buttons & (1 << i)))
 	continue;
       do_grab = !!(grab_buttons & (1 << i));
+#endif
 
       {
 	register unsigned int mods;
@@ -600,17 +706,150 @@ void focus_grab_buttons(FvwmWindow *tmp_win, Bool is_focused)
 	      XGrabButton(
 		dpy, i + 1, mods, tmp_win->Parent, True, ButtonPressMask,
 		GrabModeSync, GrabModeAsync, None, None);
+              /*  Set each FvwmWindow flag accordingly, as we grab or ungrab. */
+              tmp_win->grabbed_buttons |= (1<<i);
 	    }
 	    else
 	    {
 	      XUngrabButton(dpy, (i+1), mods, tmp_win->Parent);
+              tmp_win->grabbed_buttons &= ~(1<<i);
 	    }
 	  }
 	} /* for */
       }
     } /* for */
-    tmp_win->grabbed_buttons = grab_buttons;
     MyXUngrabServer (dpy);
+
+  }
+
+  return;
+}
+
+/* same as above, but forces to regrab buttons on the window under the pointer
+ * if necessary */
+void focus_grab_buttons_on_pointer_window(void)
+{
+  Window w;
+  FvwmWindow *tmp_win;
+
+  if (!XQueryPointer(
+	dpy, Scr.Root, &JunkRoot, &w, &JunkX, &JunkY, &JunkX, &JunkY,
+	&JunkMask))
+  {
+    /* pointer is not on this screen */
+    return;
+  }
+  if (XFindContext(dpy, w, FvwmContext, (caddr_t *) &tmp_win) == XCNOENT)
+  {
+    /* pointer is not over a window */
+    return;
+  }
+  focus_grab_buttons(tmp_win, (tmp_win == get_focus_window()));
+
+  return;
+}
+
+/* functions to access ScreenFocus, LastScreenFocus and PreviousFocus */
+FvwmWindow *get_focus_window(void)
+{
+  return ScreenFocus;
+}
+
+void set_focus_window(FvwmWindow *fw)
+{
+  ScreenFocus = fw;
+
+  return;
+}
+
+FvwmWindow *get_last_screen_focus_window(void)
+{
+  return LastScreenFocus;
+}
+
+void set_last_screen_focus_window(FvwmWindow *fw)
+{
+  LastScreenFocus = fw;
+
+  return;
+}
+
+void update_last_screen_focus_window(FvwmWindow *fw)
+{
+  if (fw == LastScreenFocus)
+    LastScreenFocus = NULL;
+
+  return;
+}
+
+void set_focus_model(FvwmWindow *fw)
+{
+  if (fw->wmhints && (fw->wmhints->flags & InputHint) && fw->wmhints->input)
+  {
+    if (WM_TAKES_FOCUS(fw))
+    {
+      fw->focus_model = FM_LOCALLY_ACTIVE;
+    }
+    else
+    {
+      fw->focus_model = FM_PASSIVE;
+    }
+  }
+  else
+  {
+    if (WM_TAKES_FOCUS(fw))
+    {
+      fw->focus_model = FM_GLOBALLY_ACTIVE;
+    }
+    else
+    {
+      fw->focus_model = FM_NO_INPUT;
+    }
+  }
+
+  return;
+}
+
+/* This function is part of a hack to make focus handling work better with
+ * applications that use the passive focus model but manage focus in their own
+ * sub windows and should thus use the locally active focus model instead.
+ * There are many examples like netscape or ddd. */
+void refresh_focus(FvwmWindow *fw)
+{
+  Bool do_refresh = False;
+
+  if (fw == NULL || fw != get_focus_window() || HAS_NEVER_FOCUS(fw))
+  {
+    /* only refresh the focus on the currently focused window */
+    return;
+  }
+
+  /* only refresh the focus for windows with the passive focus model so that
+   * we don't disturb focus handling more than necessary */
+  switch (fw->focus_model)
+  {
+  case FM_PASSIVE:
+    do_refresh = True;
+    break;
+  case FM_NO_INPUT:
+  case FM_GLOBALLY_ACTIVE:
+  case FM_LOCALLY_ACTIVE:
+  default:
+    do_refresh = False;
+    break;
+  }
+  if (do_refresh)
+  {
+    XWindowAttributes winattrs;
+
+    MyXGrabServer(dpy);
+    if (XGetWindowAttributes(dpy, fw->w, &winattrs))
+    {
+      XSelectInput(dpy, fw->w, winattrs.your_event_mask & ~FocusChangeMask);
+      FOCUS_SET(fw->w);
+      XSelectInput(dpy, fw->w, winattrs.your_event_mask);
+    }
+    MyXUngrabServer(dpy);
   }
 
   return;
