@@ -31,6 +31,7 @@
 
 #include <stdio.h>
 #include <stdarg.h>
+#include <time.h>
 
 /* To add timing info to debug output, #define this: */
 #ifdef FVWM_DEBUG_TIME
@@ -147,16 +148,8 @@ Bool GrabEm(int cursor, int grab_context)
     /* cannot happen */
     return False;
   default:
-    if (!Scr.Focus || do_accept_input_focus(Scr.Focus))
-    {
-      /* But do not grab if the window does not accept input focus from the
-       * window manager */
-      if(Scr.PreviousFocus == NULL)
-	Scr.PreviousFocus = Scr.Focus;
-      DeleteFocus(0);
-    }
     grab_win = Scr.Root;
-    rep = 500;
+    rep = NUMBER_OF_GRAB_ATTEMPTS;
     break;
   }
 
@@ -180,7 +173,7 @@ Bool GrabEm(int cursor, int grab_context)
     default:
       /* If you go too fast, other windows may not get a change to release
        * any grab that they have. */
-      usleep(10000);
+      usleep(1000 * TIME_BETWEEN_GRAB_ATTEMPTS);
       i++;
       break;
     }
@@ -207,19 +200,19 @@ print_grab_stats("grabbed");
  * UnGrab the pointer
  *
  ****************************************************************************/
-void UngrabEm(int ungrab_context)
+Bool UngrabEm(int ungrab_context)
 {
   if (ungrab_context <= GRAB_ALL || ungrab_context >= GRAB_MAXVAL)
   {
     fvwm_msg(
       ERR, "UngrabEm", "Bug: Called with illegal context %d", ungrab_context);
-    return;
+    return False;
   }
 
   if (grab_count[ungrab_context] == 0 || grab_count[GRAB_ALL] == 0)
   {
     /* context is not grabbed */
-    return;
+    return False;
   }
 
   XSync(dpy,0);
@@ -234,18 +227,11 @@ void UngrabEm(int ungrab_context)
     {
     case GRAB_NORMAL:
     case GRAB_BUSY:
+    case GRAB_MENU:
       if (grab_count[GRAB_BUSYMENU] > 0)
 	new_cursor = CRS_WAIT;
       else if (grab_count[GRAB_BUSY] > 0)
 	new_cursor = CRS_WAIT;
-      else
-	new_cursor = None;
-      break;
-    case GRAB_MENU:
-      if (grab_count[GRAB_BUSY] > 0)
-	new_cursor = CRS_WAIT;
-      else if (grab_count[GRAB_BUSYMENU] > 0)
-	new_cursor = CRS_MENU;
       else if (grab_count[GRAB_MENU] > 0)
 	new_cursor = CRS_MENU;
       else
@@ -273,17 +259,10 @@ print_grab_stats("-restore");
 print_grab_stats("-ungrab");
 #endif
     XUngrabPointer(dpy, CurrentTime);
-    if (Scr.PreviousFocus && ungrab_context != GRAB_BUSY)
-    {
-      /* if the window still exists, focus on it */
-      if (Scr.PreviousFocus->w)
-	SetFocusWindow(Scr.PreviousFocus, 0);
-      Scr.PreviousFocus = NULL;
-    }
   }
   XSync(dpy,0);
 
-  return;
+  return True;
 }
 
 #ifndef fvwm_msg /* Some ports (i.e. VMS) define their own version */
@@ -319,8 +298,8 @@ void fvwm_msg(fvwm_msg_type type, char *id, char *msg, ...)
   time_val = (unsigned int)times(&not_used_tms); /* get clock ticks */
   time_taken = time_val - prev_time;
   prev_time = time_val;
-  sprintf(buffer, "%.2d:%.2d:%.2d %ld",
-          t_ptr->tm_hour, t_ptr->tm_min, t_ptr->tm_sec,time_taken);
+  sprintf(buffer, "%.2d:%.2d:%.2d %6ld",
+          t_ptr->tm_hour, t_ptr->tm_min, t_ptr->tm_sec, time_taken);
 #endif
 
   if (Scr.NumberOfScreens > 1)
@@ -404,9 +383,11 @@ void NewFontAndColor(Font newfont, Pixel color, Pixel backcolor)
  * shortcuts by warping the pointer.
  *
  ****************************************************************************/
-void Keyboard_shortcuts(XEvent *Event, FvwmWindow *fw, int ReturnEvent)
+void Keyboard_shortcuts(
+  XEvent *Event, FvwmWindow *fw, int *x_defect, int *y_defect, int ReturnEvent)
 {
-  int x_move_size = 0, y_move_size = 0;
+  int x_move_size = 0;
+  int y_move_size = 0;
 
   if (fw)
   {
@@ -414,7 +395,8 @@ void Keyboard_shortcuts(XEvent *Event, FvwmWindow *fw, int ReturnEvent)
     y_move_size = fw->hints.height_inc;
   }
   fvwmlib_keyboard_shortcuts(
-    dpy, Scr.screen, Event, x_move_size, y_move_size, ReturnEvent);
+    dpy, Scr.screen, Event, x_move_size, y_move_size, x_defect, y_defect,
+    ReturnEvent);
 
   return;
 }
@@ -488,3 +470,21 @@ Bool move_into_rectangle(rectangle *move_rec, rectangle *target_rec)
 
   return has_changed;
 }
+
+Bool intersect_xrectangles(XRectangle *r1, XRectangle *r2)
+{
+  int x1 = max(r1->x, r2->x);
+  int y1 = max(r1->y, r2->y);
+  int x2 = min(r1->x + r1->width, r2->x + r2->width);
+  int y2 = min(r1->y + r1->height, r2->y + r2->height);
+
+  r1->x = x1;
+  r1->y = y1;
+  r1->width = x2 - x1;
+  r1->height = y2 - y1;
+
+  if (x2 > x1 && y2 > y1)
+    return True;
+  else
+    return False;
+}
