@@ -14,15 +14,6 @@
  */
 
 /****************************************************************************
- * Changed 10/06/97 by dje:
- * Change single IconBox into chain of IconBoxes.
- * Allow IconBox to be specified using X Geometry string.
- * Parse optional IconGrid.
- * Parse optional IconFill.
- * Use macros to make parsing more uniform, and easier to read.
- * Rewrote AddToList without tons of arg passing and merging.
- * Added a few comments.
- *
  * This module was all original code
  * by Rob Nation
  * Copyright 1993, Robert Nation
@@ -39,6 +30,7 @@
 #include <stdio.h>
 
 #include "libs/fvwmlib.h"
+#include "libs/FScreen.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -205,7 +197,8 @@ static void copy_icon_boxes(icon_boxes **pdest, icon_boxes *src)
 }
 
 /* Check word after IconFill to see if its "Top,Bottom,Left,Right" */
-static int Get_TBLR(char *token, unsigned char *IconFill) {
+static int Get_TBLR(char *token, unsigned char *IconFill)
+{
   /* init */
   if (StrEquals(token, "B") ||
       StrEquals(token, "BOT")||
@@ -254,8 +247,10 @@ static int Get_TBLR(char *token, unsigned char *IconFill) {
  *  Inputs:
  *      merged_style - style resulting from the merge
  *      add_style    - the style to be added into the merge_style
- *      do_free      - free allocated parts of merge_style that are replaced
- +                     from add_style
+ *      do_free_src_and_alloc_copy
+ *                   - free allocated parts of merge_style that are replaced
+ *                     from add_style.  Create a copy of of the replaced
+ *                     styles in allocated memory.
  *
  *  Note:
  *      The only trick here is that on and off flags/buttons are
@@ -264,7 +259,8 @@ static int Get_TBLR(char *token, unsigned char *IconFill) {
  ***********************************************************************/
 
 static void merge_styles(
-  window_style *merged_style, window_style *add_style, Bool do_free)
+  window_style *merged_style, window_style *add_style,
+  Bool do_free_src_and_alloc_copy)
 {
   int i;
   char *merge_flags;
@@ -276,7 +272,7 @@ static void merge_styles(
 
   if (add_style->flag_mask.has_icon)
   {
-    if (do_free)
+    if (do_free_src_and_alloc_copy)
     {
       SAFEFREE(SGET_ICON_NAME(*merged_style));
       SSET_ICON_NAME(*merged_style, (SGET_ICON_NAME(*add_style)) ?
@@ -290,7 +286,7 @@ static void merge_styles(
 #ifdef MINI_ICONS
   if (add_style->flag_mask.has_mini_icon)
   {
-    if (do_free)
+    if (do_free_src_and_alloc_copy)
     {
       SAFEFREE(SGET_MINI_ICON_NAME(*merged_style));
       SSET_MINI_ICON_NAME(*merged_style, (SGET_MINI_ICON_NAME(*add_style)) ?
@@ -305,7 +301,7 @@ static void merge_styles(
 #ifdef USEDECOR
   if (add_style->flag_mask.has_decor)
   {
-    if (do_free)
+    if (do_free_src_and_alloc_copy)
     {
       SAFEFREE(SGET_DECOR_NAME(*merged_style));
       SSET_DECOR_NAME(*merged_style, (SGET_DECOR_NAME(*add_style)) ?
@@ -319,7 +315,7 @@ static void merge_styles(
 #endif
   if (SFHAS_ICON_FONT(*add_style))
   {
-    if (do_free)
+    if (do_free_src_and_alloc_copy)
     {
       SAFEFREE(SGET_ICON_FONT(*merged_style));
       SSET_ICON_FONT(*merged_style, (SGET_ICON_FONT(*add_style)) ?
@@ -332,7 +328,7 @@ static void merge_styles(
   }
   if (SFHAS_WINDOW_FONT(*add_style))
   {
-    if (do_free)
+    if (do_free_src_and_alloc_copy)
     {
       SAFEFREE(SGET_WINDOW_FONT(*merged_style));
       SSET_WINDOW_FONT(*merged_style, (SGET_WINDOW_FONT(*add_style)) ?
@@ -343,16 +339,19 @@ static void merge_styles(
       SSET_WINDOW_FONT(*merged_style, SGET_WINDOW_FONT(*add_style));
     }
   }
-  if(add_style->flags.use_start_on_desk)
-  /*  RBW - 11/02/1998  */
+  if (add_style->flags.use_start_on_desk)
   {
     SSET_START_DESK(*merged_style, SGET_START_DESK(*add_style));
     SSET_START_PAGE_X(*merged_style, SGET_START_PAGE_X(*add_style));
     SSET_START_PAGE_Y(*merged_style, SGET_START_PAGE_Y(*add_style));
   }
+  if (add_style->flags.use_start_on_screen)
+  {
+    SSET_START_SCREEN(*merged_style, SGET_START_SCREEN(*add_style));
+  }
   if (add_style->flag_mask.has_color_fore)
   {
-    if (do_free)
+    if (do_free_src_and_alloc_copy)
     {
       SAFEFREE(SGET_FORE_COLOR_NAME(*merged_style));
       SSET_FORE_COLOR_NAME(*merged_style, (SGET_FORE_COLOR_NAME(*add_style)) ?
@@ -365,7 +364,7 @@ static void merge_styles(
   }
   if (add_style->flag_mask.has_color_back)
   {
-    if (do_free)
+    if (do_free_src_and_alloc_copy)
     {
       SAFEFREE(SGET_BACK_COLOR_NAME(*merged_style));
       SSET_BACK_COLOR_NAME(*merged_style, (SGET_BACK_COLOR_NAME(*add_style)) ?
@@ -378,11 +377,12 @@ static void merge_styles(
   }
   if (add_style->flag_mask.has_color_fore_hi)
   {
-    if (do_free)
+    if (do_free_src_and_alloc_copy)
     {
       SAFEFREE(SGET_FORE_COLOR_NAME_HI(*merged_style));
       SSET_FORE_COLOR_NAME_HI(
-	*merged_style, SGET_FORE_COLOR_NAME_HI(*add_style));
+        *merged_style, (SGET_FORE_COLOR_NAME_HI(*add_style)) ?
+        safestrdup(SGET_FORE_COLOR_NAME_HI(*add_style)) : NULL);
     }
     else
     {
@@ -392,11 +392,12 @@ static void merge_styles(
   }
   if (add_style->flag_mask.has_color_back_hi)
   {
-    if (do_free)
+    if (do_free_src_and_alloc_copy)
     {
       SAFEFREE(SGET_BACK_COLOR_NAME_HI(*merged_style));
       SSET_BACK_COLOR_NAME_HI(
-	*merged_style, SGET_BACK_COLOR_NAME_HI(*add_style));
+        *merged_style, (SGET_BACK_COLOR_NAME_HI(*add_style)) ?
+        safestrdup(SGET_BACK_COLOR_NAME_HI(*add_style)) : NULL);
     }
     else
     {
@@ -404,20 +405,20 @@ static void merge_styles(
 	*merged_style, SGET_BACK_COLOR_NAME_HI(*add_style));
     }
   }
-  if(add_style->flags.has_border_width)
+  if (add_style->flags.has_border_width)
   {
     SSET_BORDER_WIDTH(*merged_style, SGET_BORDER_WIDTH(*add_style));
   }
-  if(add_style->flags.has_handle_width)
+  if (add_style->flags.has_handle_width)
   {
     SSET_HANDLE_WIDTH(*merged_style, SGET_HANDLE_WIDTH(*add_style));
   }
-  if(add_style->flags.has_max_window_size)
+  if (add_style->flags.has_max_window_size)
   {
     SSET_MAX_WINDOW_WIDTH(*merged_style, SGET_MAX_WINDOW_WIDTH(*add_style));
     SSET_MAX_WINDOW_HEIGHT(*merged_style, SGET_MAX_WINDOW_HEIGHT(*add_style));
   }
-  if(add_style->flags.has_window_shade_steps)
+  if (add_style->flags.has_window_shade_steps)
   {
     SSET_WINDOW_SHADE_STEPS(*merged_style, SGET_WINDOW_SHADE_STEPS(*add_style));
   }
@@ -428,7 +429,7 @@ static void merge_styles(
   {
     /* If style has iconboxes */
     /* copy it */
-    if (do_free)
+    if (do_free_src_and_alloc_copy)
     {
       remove_icon_boxes_from_style(merged_style);
       copy_icon_boxes(
@@ -724,6 +725,7 @@ static Bool remove_all_of_style_from_list(char *style_ref)
 void CMD_DestroyStyle(F_CMD_ARGS)
 {
   char *name;
+  FvwmWindow *t;
 
   /* parse style name */
   name = PeekToken(action, &action);
@@ -738,6 +740,17 @@ void CMD_DestroyStyle(F_CMD_ARGS)
     /* compact the current list of styles */
     Scr.flags.do_need_style_list_update = 1;
   }
+  /* mark windows for update */
+  for (t = Scr.FvwmRoot.next; t != NULL && t != &Scr.FvwmRoot; t = t->next)
+  {
+    if (matchWildcards(name, t->class.res_class) == TRUE ||
+	matchWildcards(name, t->class.res_name) == TRUE ||
+	matchWildcards(name, t->name) == TRUE)
+    {
+      SET_STYLE_DELETED(t, 1);
+      Scr.flags.do_need_window_update = 1;
+    }
+  }
 }
 
 
@@ -753,13 +766,6 @@ void CMD_DestroyStyle(F_CMD_ARGS)
  *      tmp_win - FvwWindow structure to match against
  *      styles - callers return area
  *
- *  Changes:
- *      dje 10/06/97 test for NULL class removed, can't happen.
- *      use merge subroutine instead of coding merges 3 times.
- *      Use structure to return values, not many, many args
- *      and return value.
- *      Point at iconboxes chain, not single iconboxes elements.
- *
  ***********************************************************************/
 void lookup_style(FvwmWindow *tmp_win, window_style *styles)
 {
@@ -825,12 +831,10 @@ void CMD_Style(F_CMD_ARGS)
   int butt;
   int num;
   int i;
-  /*  RBW - 11/02/1998  */
   int tmpno[3] = { -1, -1, -1 };
   int val[4];
   int spargs = 0;
   Bool found;
-/**/
   /* temp area to build name list */
   window_style *ptmpstyle;
   /* which current boxes to chain to */
@@ -974,16 +978,23 @@ void CMD_Style(F_CMD_ARGS)
         else if (StrEquals(token, "BackingStore"))
         {
           found = True;
-	  ptmpstyle->flags.use_backing_store = 1;
-	  ptmpstyle->flag_mask.use_backing_store = 1;
-	  ptmpstyle->change_mask.use_backing_store = 1;
+	  ptmpstyle->flags.use_backing_store = BACKINGSTORE_ON;
+	  ptmpstyle->flag_mask.use_backing_store = BACKINGSTORE_MASK;
+	  ptmpstyle->change_mask.use_backing_store = BACKINGSTORE_MASK;
         }
         else if (StrEquals(token, "BackingStoreOff"))
         {
           found = True;
-	  ptmpstyle->flags.use_backing_store = 0;
-	  ptmpstyle->flag_mask.use_backing_store = 1;
-	  ptmpstyle->change_mask.use_backing_store = 1;
+	  ptmpstyle->flags.use_backing_store = BACKINGSTORE_OFF;
+	  ptmpstyle->flag_mask.use_backing_store = BACKINGSTORE_MASK;
+	  ptmpstyle->change_mask.use_backing_store = BACKINGSTORE_MASK;
+        }
+        else if (StrEquals(token, "BackingStoreWindowDefault"))
+        {
+          found = True;
+	  ptmpstyle->flags.use_backing_store = BACKINGSTORE_DEFAULT;
+	  ptmpstyle->flag_mask.use_backing_store = BACKINGSTORE_MASK;
+	  ptmpstyle->change_mask.use_backing_store = BACKINGSTORE_MASK;
         }
 	else if (StrEquals(token, "BorderColorset"))
 	{
@@ -1519,6 +1530,7 @@ void CMD_Style(F_CMD_ARGS)
         else if (StrEquals(token, "IconBox"))
         {
           icon_boxes *IconBoxes = NULL;
+	  Bool is_screen_given = False;
 
 	  found = True;
 	  token = PeekToken(rest, NULL);
@@ -1552,11 +1564,22 @@ void CMD_Style(F_CMD_ARGS)
           IconBoxes = (icon_boxes *)safemalloc(sizeof(icon_boxes));
 	  /* clear it */
           memset(IconBoxes, 0, sizeof(icon_boxes));
+	  IconBoxes->IconScreen = FSCREEN_GLOBAL;
 	  /* init grid x */
           IconBoxes->IconGrid[0] = 3;
 	  /* init grid y */
           IconBoxes->IconGrid[1] = 3;
 
+	  /* check for screen (for 4 numbers) */
+	  if (StrEquals(token, "screen"))
+	  {
+	    is_screen_given = True;
+	    token = PeekToken(rest, &rest); /* skip screen */
+	    token = PeekToken(rest, &rest); /* get the screen spec */
+	    IconBoxes->IconScreen =
+	      FScreenGetScreenArgument(token, FSCREEN_SPEC_PRIMARY);
+	  }
+
           /* try for 4 numbers x y x y */
 	  num = GetIntegerArguments(rest, NULL, val, 4);
 
@@ -1572,16 +1595,19 @@ void CMD_Style(F_CMD_ARGS)
 	      /* If leading minus sign */
 	      token = PeekToken(rest, &rest);
               if (token[0] == '-') {
-		/* if a width */
-                if (i == 0 || i == 2) {
-                  IconBoxes->IconBox[i] += Scr.MyDisplayWidth;
-                } else {
-                  /* it must be a height */
-                  IconBoxes->IconBox[i] += Scr.MyDisplayHeight;
-                } /* end width/height */
+                IconBoxes->IconSign[i]='-';
               } /* end leading minus sign */
             }
             /* Note: here there is no test for valid co-ords, use geom */
+	  } else if  (is_screen_given) {
+	    /* screen-spec is given but not 4 numbers */
+	    fvwm_msg(ERR,"CMD_Style",
+	      "IconBox requires 4 numbers if screen is given! Invalid: <%s>.",
+	      token);
+		/* Drop the box */
+                free(IconBoxes);
+		/* forget about it */
+                IconBoxes = 0;
           } else {
 	    /* Not 4 numeric args dje */
 	    /* bigger than =32767x32767+32767+32767 */
@@ -1596,12 +1622,9 @@ void CMD_Style(F_CMD_ARGS)
 	    l = strlen(token);
 	    if (l > 0 && l < 24) {
 	      /* if word found, not too long */
-              geom_flags=XParseGeometry(token,
-                                        &IconBoxes->IconBox[0],
-					/* x/y */
-                                        &IconBoxes->IconBox[1],
-					/* width/ht */
-                                        &width, &height);
+              geom_flags = FScreenParseGeometryWithScreen(
+		token, &IconBoxes->IconBox[0], &IconBoxes->IconBox[1],
+		&width, &height,&IconBoxes->IconScreen);
               if (width == 0) {
 		/* zero width is invalid */
                 fvwm_msg(ERR,"CMD_Style",
@@ -1615,19 +1638,19 @@ void CMD_Style(F_CMD_ARGS)
 		/* got valid iconbox geom */
                 if (geom_flags & XNegative) {
                   IconBoxes->IconBox[0] =
-		    /* screen width */
-		    Scr.MyDisplayWidth
 		    /* neg x coord */
-                    + IconBoxes->IconBox[0]
+                    IconBoxes->IconBox[0]
                     - width -2;
+                  IconBoxes->IconSign[0]='-'; /* save for later */
+                  IconBoxes->IconSign[2]='-'; /* save for later */
                 }
                 if (geom_flags & YNegative) {
                   IconBoxes->IconBox[1] =
-		    /* scr height */
-		    Scr.MyDisplayHeight
 		    /* neg y coord */
-                    + IconBoxes->IconBox[1]
+                    IconBoxes->IconBox[1]
                     - height -2;
+                  IconBoxes->IconSign[1]='-'; /* save for later */
+                  IconBoxes->IconSign[3]='-'; /* save for later */
                 }
 		/* x + wid = right x */
                 IconBoxes->IconBox[2] = width + IconBoxes->IconBox[0];
@@ -2007,6 +2030,27 @@ void CMD_Style(F_CMD_ARGS)
           ptmpstyle->flag_mask.use_no_pposition = 1;
           ptmpstyle->change_mask.use_no_pposition = 1;
         }
+        else if (StrEquals(token, "NoUSPosition"))
+        {
+	  found = True;
+          ptmpstyle->flags.use_no_usposition = 1;
+          ptmpstyle->flag_mask.use_no_usposition = 1;
+          ptmpstyle->change_mask.use_no_usposition = 1;
+        }
+        else if (StrEquals(token, "NoTransientPPosition"))
+        {
+	  found = True;
+          ptmpstyle->flags.use_no_transient_pposition = 1;
+          ptmpstyle->flag_mask.use_no_transient_pposition = 1;
+          ptmpstyle->change_mask.use_no_transient_pposition = 1;
+        }
+        else if (StrEquals(token, "NoTransientUSPosition"))
+        {
+	  found = True;
+          ptmpstyle->flags.use_no_transient_usposition = 1;
+          ptmpstyle->flag_mask.use_no_transient_usposition = 1;
+          ptmpstyle->change_mask.use_no_transient_usposition = 1;
+        }
         else if (StrEquals(token, "NoIconPosition"))
         {
 	  found = True;
@@ -2290,7 +2334,6 @@ void CMD_Style(F_CMD_ARGS)
         else if (StrEquals(token, "STARTSONDESK"))
         {
 	  found = True;
-	  /*  RBW - 11/02/1998  */
 	  spargs = GetIntegerArguments(rest, NULL, tmpno, 1);
           if (spargs == 1)
 	  {
@@ -2306,11 +2349,8 @@ void CMD_Style(F_CMD_ARGS)
 	    fvwm_msg(ERR,"CMD_Style",
 		     "bad StartsOnDesk arg: %s", rest);
 	  }
-          /**/
         }
-	/*  RBW - 11/02/1998
-	 *  StartsOnPage is like StartsOnDesk-Plus
-	 */
+	/* StartsOnPage is like StartsOnDesk-Plus */
 	else if (StrEquals(token, "STARTSONPAGE"))
 	{
 	  found = True;
@@ -2367,7 +2407,24 @@ void CMD_Style(F_CMD_ARGS)
 	  ptmpstyle->flag_mask.use_start_on_page_for_transient = 1;
 	  ptmpstyle->change_mask.use_start_on_page_for_transient = 1;
 	}
-	/**/
+	else if (StrEquals(token, "StartsOnScreen"))
+        {
+	  found = True;
+	  if (rest)
+	  {
+	    tmpno[0] = FScreenGetScreenArgument(rest, 'c');
+	    ptmpstyle->flags.use_start_on_screen = 1;
+	    ptmpstyle->flag_mask.use_start_on_screen = 1;
+	    ptmpstyle->change_mask.use_start_on_screen = 1;
+	    SSET_START_SCREEN(*ptmpstyle, tmpno[0]);
+	  }
+          else
+	  {
+	    ptmpstyle->flags.use_start_on_screen = 0;
+	    ptmpstyle->flag_mask.use_start_on_screen = 1;
+	    ptmpstyle->change_mask.use_start_on_screen = 1;
+	  }
+        }
 	else if (StrEquals(token, "STARTSANYWHERE"))
 	{
 	  found = True;
@@ -2472,6 +2529,27 @@ void CMD_Style(F_CMD_ARGS)
           ptmpstyle->flag_mask.use_no_pposition = 1;
           ptmpstyle->change_mask.use_no_pposition = 1;
         }
+        else if (StrEquals(token, "UseUSPosition"))
+        {
+	  found = True;
+          ptmpstyle->flags.use_no_usposition = 0;
+          ptmpstyle->flag_mask.use_no_usposition = 1;
+          ptmpstyle->change_mask.use_no_usposition = 1;
+        }
+        else if (StrEquals(token, "UseTransientPPosition"))
+        {
+	  found = True;
+          ptmpstyle->flags.use_no_transient_pposition = 0;
+          ptmpstyle->flag_mask.use_no_transient_pposition = 1;
+          ptmpstyle->change_mask.use_no_transient_pposition = 1;
+        }
+        else if (StrEquals(token, "UseTransientUSPosition"))
+        {
+	  found = True;
+          ptmpstyle->flags.use_no_transient_usposition = 0;
+          ptmpstyle->flag_mask.use_no_transient_usposition = 1;
+          ptmpstyle->change_mask.use_no_transient_usposition = 1;
+        }
         else if (StrEquals(token, "UseIconPosition"))
         {
 	  found = True;
@@ -2480,7 +2558,7 @@ void CMD_Style(F_CMD_ARGS)
 	  SCSET_USE_ICON_POSITION_HINT(*ptmpstyle, 1);
         }
 #ifdef USEDECOR
-        if (StrEquals(token, "UseDecor"))
+        else if (StrEquals(token, "UseDecor"))
         {
 	  found = True;
 	  SAFEFREE(SGET_DECOR_NAME(*ptmpstyle));
@@ -2634,7 +2712,7 @@ void CMD_Style(F_CMD_ARGS)
   {
     /* merge with previous style */
     merge_styles(last_style_in_list, ptmpstyle, True);
-    free(SGET_NAME(*ptmpstyle));
+    free_style(ptmpstyle);
     free(ptmpstyle);
   }
   else
@@ -2655,7 +2733,7 @@ void check_window_style_change(
   char *sc;
 
   lookup_style(t, ret_style);
-  if (ret_style->has_style_changed == 0)
+  if (!ret_style->has_style_changed && !IS_STYLE_DELETED(t))
   {
     /* nothing to do */
     return;
@@ -2674,6 +2752,14 @@ void check_window_style_change(
     wf[i] = (wf[i] & ~sc[i]) | (sf[i] & sc[i]);
   }
 
+  if (IS_STYLE_DELETED(t))
+  {
+    /* update all styles */
+    memset(flags, 0xff, sizeof(*flags));
+    SET_STYLE_DELETED(t, 0);
+    return;
+  }
+
   /*
    * is_sticky
    * is_icon_sticky
@@ -2759,8 +2845,12 @@ void check_window_style_change(
    *   do_place_smart
    *   do_start_lowered
    *   use_no_pposition
+   *   use_no_usposition
+   *   use_no_transient_pposition
+   *   use_no_transient_usposition
    *   use_start_on_desk
    *   use_start_on_page_for_transient
+   *   use_start_on_screen
    *   manual_placement_honors_starts_on_page
    *   capture_honors_starts_on_page
    *   recapture_honors_starts_on_page
@@ -2803,7 +2893,6 @@ void check_window_style_change(
     flags->do_update_modules_flags = True;
   }
 
-
   /*
    *   has_icon_boxes
    */
