@@ -53,12 +53,11 @@
 #ifdef XPM
 #include <X11/xpm.h>
 #endif
-#ifdef SHAPE
-#include <X11/extensions/shape.h>
-#endif
 
 #include "libs/defaults.h"
 #include "libs/fvwmlib.h"
+#include "libs/FScreen.h"
+#include "libs/FShape.h"
 #include "libs/Grab.h"
 #include "fvwm/fvwm.h"
 #include "libs/Module.h"
@@ -75,7 +74,7 @@
 #define MW_EVENTS   (ExposureMask |\
 		     StructureNotifyMask |\
 		     ButtonReleaseMask | ButtonPressMask |\
-		     KeyReleaseMask | KeyPressMask)
+		     KeyReleaseMask | KeyPressMask | ButtonMotionMask)
 /* SW_EVENTS are for swallowed windows... */
 #define SW_EVENTS   (PropertyChangeMask | StructureNotifyMask |\
 		     ResizeRedirectMask | SubstructureNotifyMask)
@@ -133,7 +132,6 @@ static void HandlePanelPress(button_info *b);
 
 Display *Dpy;
 Window Root;
-static GC trans_gc = NULL;
 Window MyWindow;
 char *MyName;
 static int screen;
@@ -172,6 +170,7 @@ Bool is_transient = 0;
 Bool is_transient_panel = 0;
 
 button_info *CurrentButton = NULL;
+Bool is_pointer_in_current_button = False;
 int fd[2];
 
 button_info *UberButton=NULL;
@@ -358,6 +357,10 @@ static void DeadPipeCleanup(void)
 	  XMoveWindow(Dpy,swin,b->x,b->y);
 	  XResizeWindow(Dpy,swin,b->w,b->h);
 	  XSetWindowBorderWidth(Dpy,swin,b->bw);
+	  if ((b->flags & b_Panel))
+	  {
+	    XMapWindow(Dpy, swin);
+	  }
 	}
       }
 #ifdef DEBUG_HANGON
@@ -366,6 +369,7 @@ static void DeadPipeCleanup(void)
 #endif
     }
   }
+  XSync(Dpy, 0);
   MyXUngrabServer(Dpy); /* We're through */
 
   /* Hey, we have to free the pictures too! */
@@ -504,7 +508,6 @@ char *GetButtonAction(button_info *b,int n)
   return expand_action(b->action[n], b);
 }
 
-#ifdef SHAPE
 /**
 *** SetTransparentBackground()
 *** use the Shape extension to create a transparent background.
@@ -512,75 +515,140 @@ char *GetButtonAction(button_info *b,int n)
 **/
 void SetTransparentBackground(button_info *ub,int w,int h)
 {
-  Pixmap pmap_mask;
-  button_info *b;
-  Window root_return;
-  int x_return, y_return;
-  unsigned int width_return, height_return;
-  unsigned int border_width_return;
-  unsigned int depth_return;
-  int number, i;
-  XFontStruct *font;
-
-  pmap_mask = XCreatePixmap(Dpy,MyWindow,w,h,1);
-
-  if (trans_gc == NULL)
+  if (FShapesSupported)
   {
-    XGCValues gcv;
+    Pixmap pmap_mask;
+    button_info *b;
+    Window root_return;
+    int x_return, y_return;
+    unsigned int width_return, height_return;
+    unsigned int border_width_return;
+    unsigned int depth_return;
+    int number, i;
+    XFontStruct *font;
+    Region shape_r;
+    static GC trans_gc = NULL;
 
-    /* create a GC for doing transparency */
-    trans_gc = fvwmlib_XCreateGC(Dpy,pmap_mask,(unsigned long)0,&gcv);
-  }
+    pmap_mask = XCreatePixmap(Dpy,MyWindow,w,h,1);
+    if (pmap_mask == None)
+      return;
+    shape_r = XCreateRegion();
+    if (trans_gc == NULL)
+    {
+      XGCValues gcv;
 
-  XSetForeground(Dpy,trans_gc,0);
-  XFillRectangle(Dpy,pmap_mask,trans_gc,0,0,w,h);
-  XSetForeground(Dpy,trans_gc,1);
+      /* create a GC for doing transparency */
+      trans_gc = fvwmlib_XCreateGC(Dpy,pmap_mask,(unsigned long)0,&gcv);
+      if (trans_gc == NULL)
+	return;
+    }
+    XSetClipMask(Dpy, trans_gc, None);
+    XSetForeground(Dpy, trans_gc, 0);
+    XFillRectangle(Dpy, pmap_mask, trans_gc, 0, 0, w, h);
+    XSetForeground(Dpy, trans_gc, 1);
 
-  /*
-   * if button has an icon, draw a rect with the same size as the icon
-   * (or the mask of the icon),
-   * else draw a rect with the same size as the button.
-   */
+    /*
+     * if button has an icon, draw a rect with the same size as the icon
+     * (or the mask of the icon),
+     * else draw a rect with the same size as the button.
+     */
 
-  i=-1;
-  while(NextButton(&ub,&b,&i,0))
-  {
-    if (b->flags&b_Icon &&
-	XGetGeometry(Dpy,b->IconWin,&root_return,&x_return,&y_return,
-		     &width_return,&height_return,
-		     &border_width_return,&depth_return))
+    i = -1;
+    while (NextButton(&ub,&b,&i,0))
     {
       number=buttonNum(b);
-      if (b->icon->mask == None)
+      if (b->flags&b_Icon &&
+	  XGetGeometry(Dpy,b->IconWin,&root_return,&x_return,&y_return,
+		       &width_return,&height_return,
+		       &border_width_return,&depth_return))
       {
-	XFillRectangle(Dpy,pmap_mask,trans_gc,x_return, y_return,
-		       b->icon->width,b->icon->height);
+	if (b->icon->mask == None)
+	{
+	  XFillRectangle(Dpy,pmap_mask,trans_gc,x_return, y_return,
+			 b->icon->width,b->icon->height);
+	}
+	else
+	{
+	  XCopyArea(Dpy,b->icon->mask,pmap_mask,trans_gc,0,0,
+		    b->icon->width,b->icon->height,x_return,y_return);
+	}
+      }
+      else if (buttonSwallowCount(b) == 3 && (b->flags & b_Swallow))
+      {
+	Window swin = SwallowedWindow(b);
+	XRectangle *r;
+	int count;
+	int ordering;
+	XRectangle s;
+	Bool got_geometry = False;
+
+	if (XGetGeometry(
+	      Dpy, swin, &root_return, &x_return, &y_return, &width_return,
+	      &height_return, &border_width_return, &depth_return))
+	{
+	  got_geometry = True;
+	  s.x = x_return;
+	  s.y = y_return;
+	  s.width = width_return;
+	  s.height = height_return;
+	}
+	else
+	{
+	  s.x = buttonXPos(b, number);
+	  s.y = buttonYPos(b, number);
+	  s.width = buttonWidth(b);
+	  s.height = buttonHeight(b);
+	}
+
+	/* get set of shape rectangles - this is all very inefficient, but I do
+	 * not see another way to do it properly */
+	if (got_geometry &&
+	    (r = FShapeGetRectangles(
+	      Dpy, swin, FShapeBounding, &count, &ordering)))
+	{
+	  int i;
+
+	  /* merge the rectangles with the shape region */
+	  for (i = 0; i < count ; i++)
+	  {
+	    r[i].x += s.x;
+	    r[i].y += s.y;
+	    XUnionRectWithRegion(&(r[i]), shape_r, shape_r);
+	  }
+	  XFree(r);
+	}
+	else
+	{
+	  XUnionRectWithRegion(&s, shape_r, shape_r);
+	  /* suppress compiler warning w/o shape extension */
+	  ordering = 0;
+	}
       }
       else
       {
-	XCopyArea(Dpy,b->icon->mask,pmap_mask,trans_gc,0,0,
-		  b->icon->width,b->icon->height,x_return,y_return);
+	XFillRectangle(Dpy,pmap_mask,trans_gc,buttonXPos(b,number),
+		       buttonYPos(b,number),
+		       buttonWidth(b),buttonHeight(b));
+      }
+
+      /* handle button's title */
+      font=buttonFont(b);
+      if(b->flags&b_Title && font)
+      {
+	XSetFont(Dpy,trans_gc,font->fid);
+	DrawTitle(b,pmap_mask,trans_gc);
       }
     }
-    else
-    {
-      number=buttonNum(b);
-      XFillRectangle(Dpy,pmap_mask,trans_gc,buttonXPos(b,number),
-		     buttonYPos(b,number),
-		     buttonWidth(b),buttonHeight(b));
-    }
-
-    /* handle button's title */
-    font=buttonFont(b);
-    if(b->flags&b_Title && font)
-    {
-      XSetFont(Dpy,trans_gc,font->fid);
-      DrawTitle(b,pmap_mask,trans_gc);
-    }
+    XSetRegion(Dpy, trans_gc, shape_r);
+    XFillRectangle(Dpy, pmap_mask, trans_gc, 0, 0, w, h);
+    FShapeCombineMask(
+      Dpy, MyWindow, FShapeBounding, 0, 0, pmap_mask, FShapeSet);
+    XFreePixmap(Dpy, pmap_mask);
+    XDestroyRegion(shape_r);
   }
-  XShapeCombineMask(Dpy,MyWindow,ShapeBounding,0,0,pmap_mask,ShapeSet);
+
+  return;
 }
-#endif
 
 /**
 *** myErrorHandler()
@@ -598,7 +666,8 @@ int myErrorHandler(Display *dpy, XErrorEvent *event)
   }
   if((event->error_code == BadWindow) || (event->error_code == BadDrawable)
      || (event->error_code == BadMatch) || (event->request_code==X_GrabButton)
-     || (event->request_code == X_GetGeometry))
+     || (event->request_code == X_GetGeometry)
+     || (event->error_code == BadPixmap))
     return 0;
 
   PrintXErrorAndCoredump(dpy, event, MyName);
@@ -675,8 +744,8 @@ int main(int argc, char **argv)
 
   if(argc<6 || argc>10)
   {
-    fprintf(stderr,"%s v%s should only be executed by fvwm!\n",MyName,
-	    VERSION);
+    fprintf(stderr, "%s version %s should only be executed by fvwm!\n",
+	    MyName, VERSION);
     exit(1);
   }
 
@@ -696,24 +765,26 @@ int main(int argc, char **argv)
       }
     }
     else if (!is_transient && !is_transient_panel &&
-	     strcmp(argv[i], "-transient") == 0)
+	     (strcmp(argv[i], "-transient") == 0 ||
+	      strcmp(argv[i], "transient") == 0))
     {
       is_transient = 1;
     }
     else if (!is_transient && !is_transient_panel &&
-	     strcmp(argv[i], "-transientpanel") == 0)
+	     (strcmp(argv[i], "-transientpanel") == 0 ||
+	      strcmp(argv[i], "transientpanel") == 0))
     {
       is_transient_panel = 1;
     }
     else if (!has_name) /* There is a naming argument here! */
     {
       free(MyName);
-      MyName=strdup(argv[i]);
+      MyName=safestrdup(argv[i]);
       has_name = 1;
     }
     else if (!has_file) /* There is a config file here! */
     {
-      config_file=strdup(argv[i]);
+      config_file=safestrdup(argv[i]);
       has_file = 1;
     }
   }
@@ -727,8 +798,10 @@ int main(int argc, char **argv)
     exit (1);
   }
   InitPictureCMap(Dpy);
+  FScreenInit(Dpy);
   /* Initialise default colorset */
   AllocColorset(0);
+  FShapeInit(Dpy);
 
   x_fd=XConnectionNumber(Dpy);
   fd_width=GetFdWidth();
@@ -840,10 +913,11 @@ int main(int argc, char **argv)
   while(NextButton(&ub,&b,&i,0))
     ConfigureIconWindow(b);
 
-#ifdef SHAPE
-  if(UberButton->c->flags&b_TransBack)
-    SetTransparentBackground(UberButton,Width,Height);
-#endif
+  if (FShapesSupported)
+  {
+    if (UberButton->c->flags&b_TransBack)
+      SetTransparentBackground(UberButton,Width,Height);
+  }
 
   i=-1;
   ub=UberButton;
@@ -904,6 +978,21 @@ void Loop(void)
   {
     if(My_XNextEvent(Dpy,&Event))
     {
+      if (FShapesSupported)
+      {
+	if (Event.type == FShapeEventBase + FShapeNotify)
+	{
+	  FShapeEvent *sev = (FShapeEvent *) &Event;
+
+	  if (sev->kind != FShapeBounding)
+	    return;
+	  if (UberButton->c->flags & b_TransBack)
+	  {
+	    SetTransparentBackground(UberButton, Width, Height);
+	  }
+	  continue;
+	}
+      }
       switch(Event.type)
       {
       case Expose:
@@ -974,6 +1063,10 @@ void Loop(void)
 	  SetButtonSize(UberButton,Width,Height);
 	  button=-1;
 	  ub=UberButton;
+	  if (FShapesSupported && UberButton->c->flags & b_TransBack)
+	  {
+	    SetTransparentBackground(UberButton, Width, Height);
+	  }
 	  for (i = 0; i < nColorsets; i++)
 	    change_colorset(i);
 	  while(NextButton(&ub,&b,&button,0))
@@ -983,46 +1076,87 @@ void Loop(void)
       }
       break;
 
+      case MotionNotify:
+	{
+	  Bool f = is_pointer_in_current_button;
+
+	  is_pointer_in_current_button =
+	    (CurrentButton && CurrentButton ==
+	     select_button(UberButton, Event.xmotion.x, Event.xmotion.y));
+	  if (CurrentButton && is_pointer_in_current_button != f)
+	  {
+	    RedrawButton(b, 0);
+	  }
+	}
+	break;
+
       case KeyPress:
 	XLookupString(&Event.xkey,buffer,10,&keysym,0);
 	if(keysym!=XK_Return && keysym!=XK_KP_Enter && keysym!=XK_Linefeed)
 	  break;	                /* fall through to ButtonPress */
 
       case ButtonPress:
+	if (CurrentButton)
+	{
+	  b = CurrentButton;
+	  CurrentButton = 0;
+	  RedrawButton(b, 0);
+	  break;
+	}
+	if (Event.xbutton.state & DEFAULT_ALL_BUTTONS_MASK)
+	{
+	  break;
+	}
 	CurrentButton = b =
 	  select_button(UberButton,Event.xbutton.x,Event.xbutton.y);
+	is_pointer_in_current_button = True;
 
-	if(!b || !(b->flags&b_Action) ||
-	   ((act=GetButtonAction(b,Event.xbutton.button)) == NULL &&
-	    (act=GetButtonAction(b,0)) == NULL))
+	act = NULL;
+	if (!(b->flags & b_Panel) &&
+	    (!b || !(b->flags&b_Action) ||
+	     ((act=GetButtonAction(b,Event.xbutton.button)) == NULL &&
+	      (act=GetButtonAction(b,0)) == NULL)))
 	{
 	  CurrentButton=NULL;
 	  break;
 	}
-
 	RedrawButton(b,0);
-	if(strncasecmp(act,"popup",5)!=0)
+	if (!act)
 	{
-	  free(act);
-	  act = NULL;
+	  break;
+	}
+	if (act && strncasecmp(act,"popup",5) != 0)
+	{
+	  if (act)
+	  {
+	    free(act);
+	    act = NULL;
+	  }
 	  break;
 	}
 	else /* i.e. action is Popup */
+	{
 	  XUngrabPointer(Dpy,CurrentTime); /* And fall through */
+	}
 	free(act);
 	act = NULL;
+	/* fall through */
 
       case KeyRelease:
       case ButtonRelease:
-	b=select_button(UberButton,Event.xbutton.x,Event.xbutton.y);
+	if (CurrentButton == NULL || !is_pointer_in_current_button)
+	{
+	  CurrentButton = NULL;
+	  break;
+	}
+	b = select_button(UberButton,Event.xbutton.x,Event.xbutton.y);
 	act = GetButtonAction(b,Event.xbutton.button);
-
 	if (b && !act && (b->flags & b_Panel))
 	{
 	  HandlePanelPress(b);
 	  if (b->newflags.panel_mapped == 0)
 	  {
-	    if(is_transient)
+	    if (is_transient)
 	    {
 	      /* terminate if transient and panel has been unmapped */
 	      exit(0);
@@ -1037,14 +1171,15 @@ void Loop(void)
 	{
 	  if (!act)
 	    act=GetButtonAction(b,0);
-	  if(b && b==CurrentButton && act)
+	  if (b && b == CurrentButton && act)
 	  {
-	    if(strncasecmp(act,"Exec",4)==0)
+	    if (strncasecmp(act,"Exec",4) == 0 &&
+		isspace(act[4]))
 	    {
 
 	      /* Look for Exec "identifier", in which case the button
 		 stays down until window "identifier" materializes */
-	      i=4;
+	      i = 4;
 	      while(isspace((unsigned char)act[i]))
 		i++;
 	      if(act[i] == '"')
@@ -1119,10 +1254,10 @@ void Loop(void)
 	  free(act);
 	  act = NULL;
 	}
-	b=CurrentButton;
+	b = CurrentButton;
 	CurrentButton=NULL;
-	if(b)
-	  RedrawButton(b,0);
+	if (b)
+	  RedrawButton(b, 0);
 	break;
 
       case ClientMessage:
@@ -1167,6 +1302,13 @@ void Loop(void)
 	      if(!XGetWMNormalHints(Dpy,swin,b->hints,&supp))
 		b->hints->flags = 0;
 	      MakeButton(b);
+	      if (FShapesSupported)
+	      {
+		if (UberButton->c->flags & b_TransBack)
+		{
+		  SetTransparentBackground(UberButton, Width, Height);
+		}
+	      }
 	    }
 	    RedrawButton(b,1);
 	  }
@@ -1622,6 +1764,7 @@ static void HandlePanelPress(button_info *b)
   Bool is_mapped;
   XWindowAttributes xwa;
   char cmd[64];
+  button_info *tmp;
 
   if (XGetWindowAttributes(Dpy, b->PanelWin, &xwa))
     is_mapped = (xwa.map_state == IsViewable);
@@ -1643,6 +1786,7 @@ static void HandlePanelPress(button_info *b)
     XWithdrawWindow(Dpy, b->PanelWin, screen);
 
     /* now request mapping in the void and wait until it gets mapped */
+    mysizehints.flags = 0;
     XGetWMNormalHints(Dpy, b->PanelWin, &mysizehints, &supplied);
     mysizehints.flags |= USPosition;
     mysizehints.x = 32767;
@@ -1738,7 +1882,6 @@ static void HandlePanelPress(button_info *b)
 
   /* redraw our button */
   b->newflags.panel_mapped = !is_mapped;
-  RedrawButton(b, 1);
 
   /* make sure the window maps on the current desk */
   sprintf(cmd, "Silent WindowId 0x%08x MoveToDesk 0", (int)b->PanelWin);
@@ -1748,18 +1891,20 @@ static void HandlePanelPress(button_info *b)
 	      x2, y2, w2, h2,
 	      steps, b->slide_delay_ms, NULL, b->panel_flags.smooth,
 	      (b->swallow&b_NoHints) ? 0 : 1);
+  if (is_mapped)
+  {
+    XUnmapWindow(Dpy, b->PanelWin);
+  }
+  tmp = CurrentButton;
+  CurrentButton = NULL;
+  RedrawButton(b, 1);
+  CurrentButton = tmp;
   XSync(Dpy, 0);
   /* Give fvwm a chance to update the window.  Otherwise the window may end up
    * too small.  This doesn't prevent this completely, but makes it much less
    * likely. */
   usleep(250000);
 
-  if (is_mapped)
-  {
-    XUnmapWindow(Dpy, b->PanelWin);
-  }
-  RedrawButton(b, 1);
-
   return;
 }
 
@@ -1859,6 +2004,10 @@ void CreateUberButtonWindow(button_info *ub,int maxx,int maxy)
 			   mysizehints.width,mysizehints.height,0,Pdepth,
 			   InputOutput,Pvisual,
 			   CWColormap|CWBackPixmap|CWBorderPixel,&xswa);
+  if (is_transient)
+  {
+    XSetTransientForHint(Dpy, MyWindow, Root);
+  }
 
 #ifdef DEBUG_INIT
   fprintf(stderr,"colors...");
@@ -1884,8 +2033,8 @@ void CreateUberButtonWindow(button_info *ub,int maxx,int maxy)
 #endif
   XSetWMProtocols(Dpy,MyWindow,&_XA_WM_DEL_WIN,1);
 
-  myclasshints.res_name=strdup(MyName);
-  myclasshints.res_class=strdup("FvwmButtons");
+  myclasshints.res_name=safestrdup(MyName);
+  myclasshints.res_class=safestrdup("FvwmButtons");
 
   {
     XTextProperty mynametext;
@@ -2142,6 +2291,8 @@ void SpawnSome(void)
   static char first=1;
   button_info *b,*ub=UberButton;
   int button=-1;
+  char *p;
+
   if(!first)
     return;
   first=0;
@@ -2154,7 +2305,12 @@ void SpawnSome(void)
 	fprintf(stderr,"%s: Button 0x%06x did not find a \"%s\" window, %s",
 		MyName,(ushort)b,b->hangon,"spawning own\n");
 #endif
-	SendText(fd,b->spawn,0);
+	p = expand_action(b->spawn, NULL);
+	if (p)
+	{
+	  MySendText(fd, p, 0);
+	  free(p);
+	}
       }
 }
 
@@ -2242,7 +2398,7 @@ static void change_colorset(int colorset)
   return;
 }
 
-static void handle_colorset_packet(unsigned long *body)
+static void handle_config_info_packet(unsigned long *body)
 {
   char *tline, *token;
   int colorset;
@@ -2254,6 +2410,10 @@ static void handle_colorset_packet(unsigned long *body)
     colorset = LoadColorset(tline);
     change_colorset(colorset);
   }
+  else if (StrEquals(token, XINERAMA_CONFIG_STRING))
+  {
+    FScreenConfigureModule(tline);
+  }
 
   return;
 }
@@ -2278,7 +2438,6 @@ void process_message(unsigned long type,unsigned long *body)
     break;
   case M_END_WINDOWLIST:
     SpawnSome();
-    RedrawWindow(NULL);
     break;
   case M_MAP:
     swallow(body);
@@ -2289,12 +2448,12 @@ void process_message(unsigned long type,unsigned long *body)
     CheckForHangon(body);
     break;
   case M_CONFIG_INFO:
-    handle_colorset_packet((unsigned long*)body);
+    handle_config_info_packet((unsigned long*)body);
     break;
   case M_ADD_WINDOW:
   case M_CONFIGURE_WINDOW:
     cfgpacket = (void *) body;
-    if ( cfgpacket->w == MyWindow)
+    if (cfgpacket->w == MyWindow)
     {
       button_lborder = button_rborder = button_tborder = button_bborder
 	= cfgpacket->border_width;
@@ -2631,8 +2790,9 @@ void swallow(unsigned long *body)
 	return;
       }
 
-      if(p!=b->IconWinParent) /* The window has been reparented */
+      if (p != b->IconWinParent)
       {
+	/* The window has been reparented */
 	fprintf(stderr,"%s: Window 0x%lx (\"%s\") was %s (window 0x%lx)\n",
 		MyName,swin,b->hangon,"grabbed by someone else",p);
 
@@ -2675,17 +2835,20 @@ void swallow(unsigned long *body)
 	}
 	/*error checking*/
 	XReparentWindow(Dpy,swin,MyWindow,-32768,-32768);
+	XSync(Dpy, 0);
 	MyXUngrabServer(Dpy);
-	b->swallow&=~b_Count;
-	b->swallow|=3;
-	if(buttonSwallow(b)&b_UseTitle)
+	XSync(Dpy, 0);
+	usleep(50000);
+	b->swallow &= ~b_Count;
+	b->swallow |= 3;
+	if (buttonSwallow(b) & b_UseTitle)
 	{
-	  if(b->flags&b_Title)
+	  if (b->flags & b_Title)
 	    free(b->title);
-	  b->flags|=b_Title;
-	  if (XFetchName(Dpy,swin,&temp))
+	  b->flags |= b_Title;
+	  if (XFetchName(Dpy, swin, &temp))
 	  {
-	    CopyString(&b->title,temp);
+	    CopyString(&b->title, temp);
 	    XFree(temp);
 	  }
 	  else
@@ -2693,27 +2856,38 @@ void swallow(unsigned long *body)
 	    CopyString(&b->title, "");
 	  }
 	}
-	XMapWindow(Dpy,swin);
+
 	MakeButton(b);
+	XMapWindow(Dpy, swin);
+	XSync(Dpy, 0);
 
 	if (b->flags & b_Colorset)
 	{
 	  /* A short delay to give the application the chance to set the
 	   * background itself, so we can override it. If we don't do this, the
-	   * application may override out background. On the other hand it may
+	   * application may override our background. On the other hand it may
 	   * still override our background, but our chances are a bit better.
 	   */
-	  XSync(Dpy, 0);
 	  usleep(250000);
 	  XSync(Dpy, 0);
 	  change_swallowed_window_colorset(b, True);
 	}
-	RedrawButton(b,1);
+	if (FShapesSupported)
+	{
+	  if (UberButton->c->flags & b_TransBack)
+	  {
+	    SetTransparentBackground(UberButton, Width, Height);
+	    FShapeSelectInput(Dpy, swin, FShapeNotifyMask);
+	  }
+	}
+	/* Redraw and force cleaning the background to erase the old button
+	 * title. */
+	RedrawButton(b, 2);
       }
       else /* (b->flags & b_Panel) */
       {
-	b->swallow&=~b_Count;
-	b->swallow|=3;
+	b->swallow &= ~b_Count;
+	b->swallow |= 3;
 	XSelectInput(Dpy, swin, PA_EVENTS);
 	if (!XWithdrawWindow(Dpy, swin, screen))
 	{
