@@ -37,6 +37,8 @@
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
 
+#include "libs/fvwmlib.h"
+#include "libs/FScreen.h"
 #include "libs/Module.h"
 #include "libs/Colorset.h"
 #include "FvwmButtons.h"
@@ -496,10 +498,14 @@ static void ParsePanel(char **ss, byte *flags, byte *mask, char *direction,
 	  *position = SLIDE_POSITION_CENTER;
 	  break;
 	case 4: /* left */
+	  *position = SLIDE_POSITION_LEFT_TOP;
+	  break;
 	case 5: /* top  */
 	  *position = SLIDE_POSITION_LEFT_TOP;
 	  break;
 	case 6: /* right */
+	  *position = SLIDE_POSITION_RIGHT_BOTTOM;
+	  break;
 	case 7: /* bottom */
 	  *position = SLIDE_POSITION_RIGHT_BOTTOM;
 	  break;
@@ -724,6 +730,7 @@ static void ParseContainer(char **ss,button_info *b)
 	b->c->colorset = i;
 	b->c->flags |= b_Colorset;
 	AllocColorset(i);
+	s = t;
       }
       else
       {
@@ -901,11 +908,20 @@ static void ParseButton(button_info **uberb,char *s)
 	t=seekright(&s);
 	if(t && *t && (t[0] != '-' || t[1] != 0))
 	{
-	  if (b->icon_file)
-	    free(b->icon_file);
-	  b->icon_file=t;
-	  b->IconWin=None;
-	  b->flags|=b_Icon;
+	  if (b->flags & b_Swallow)
+	  {
+	    fprintf(
+	      stderr,"%s: a button can not have an icon and a swallowed window"
+	      " at the same time. Ignoring icon", MyName);
+	  }
+	  else
+	  {
+	    if (b->icon_file)
+	      free(b->icon_file);
+	    b->icon_file=t;
+	    b->IconWin=None;
+	    b->flags|=b_Icon;
+	  }
 	}
 	else
 	{
@@ -991,7 +1007,17 @@ static void ParseButton(button_info **uberb,char *s)
 	    free(b->hangon);
 	  b->hangon=t;
 	  if (is_swallow)
+	  {
+	    if (b->flags & b_Icon)
+	    {
+	      fprintf(
+		stderr,"%s: a button can not have an icon and a swallowed "
+		" window at the same time. Ignoring icon", MyName);
+	      b->flags &= ~ b_Icon;
+	    }
+
 	    b->flags |= (b_Swallow | b_Hangon);
+	  }
 	  else
 	  {
 	    b->flags |= (b_Panel | b_Hangon);
@@ -1051,7 +1077,7 @@ static void ParseButton(button_info **uberb,char *s)
           char *r;
           char *u = s;
 
-          s = GetQuotedString(s, &t, ",)", NULL, NULL, NULL);
+          s = GetQuotedString(s, &t, ",)", NULL, "(", ")");
           r = s;
           if (t && r > u + 1)
           {
@@ -1222,7 +1248,7 @@ static void ParseButton(button_info **uberb,char *s)
       free(b->hangon);
     b->hangon=seekright(&s);
     if (!b->hangon)
-      b->hangon = strdup("");
+      b->hangon = safestrdup("");
     if (tolower(*s) == 's')
       b->flags |= b_Swallow | b_Hangon;
     else
@@ -1235,7 +1261,7 @@ static void ParseButton(button_info **uberb,char *s)
     {
       if(!(buttonSwallow(b)&b_UseOld))
 	SendText(fd,s,0);
-      b->spawn=strdup(s);
+      b->spawn=safestrdup(s);
     }
   }
   else if(*s)
@@ -1318,7 +1344,7 @@ static void ParseConfigLine(button_info **ubb,char *s)
   case 2:/* Font */
     if (ub->c->font_string)
       free(ub->c->font_string);
-    CopyString(&ub->c->font_string, s);
+    CopyStringWithQuotes(&ub->c->font_string, s);
     break;
   case 3:/* Padding */
     i=sscanf(s,"%d %d",&j,&k);
@@ -1607,7 +1633,7 @@ void parse_window_geometry(char *geom)
   unsigned int width;
   unsigned int height;
 
-  flags = XParseGeometry(geom,&g_x,&g_y,&width,&height);
+  flags = FScreenParseGeometry(geom,&g_x,&g_y,&width,&height);
   UberButton->w = 0;
   UberButton->h = 0;
   if (flags&WidthValue)
@@ -1640,6 +1666,7 @@ void ParseConfiguration(button_info *ub)
     "imagepath",
     "colorlimit",
     "colorset",
+    XINERAMA_CONFIG_STRING,
     NULL
   };
 
@@ -1671,6 +1698,10 @@ void ParseConfiguration(button_info *ub)
       /* store colorset sent by fvwm */
       LoadColorset(rest);
       break;
+    case 4:
+      /* Xinerama state */
+      FScreenConfigureModule(rest);
+      break;
     }
     GetConfigLine(fd,&s);
   }
