@@ -19,10 +19,12 @@
 #include "x.h"
 #include "xmanager.h"
 
+#include <libs/fvwmlib.h>
+#include <libs/FScreen.h>
 #include <libs/Module.h>
 
 static char const rcsid[] =
-  "$Id: fvwm.c,v 1.31 2001/03/30 09:29:21 domivogt Exp $";
+  "$Id: fvwm.c,v 1.34.2.2 2002/11/10 19:54:33 domivogt Exp $";
 
 static WinData *fvwm_focus_win = NULL;
 
@@ -146,43 +148,65 @@ int win_in_viewport (WinData *win)
 {
   WinManager *manager = win->manager;
   int flag = 0;
+  int reverse = 0;
+  rectangle g;
 
   assert (manager);
 
+  if (IS_ICONIFIED(win))
+  {
+    g = win->icon_g;
+  }
+  else
+  {
+    g.x = win->x;
+    g.y = win->y;
+    g.width = win->width;
+    g.height = win->height;
+  }
   switch (manager->res) {
   case SHOW_GLOBAL:
     flag = 1;
     break;
 
+  case NO_SHOW_DESKTOP:
+    reverse = 1;
+    /* fall through to next case */
   case SHOW_DESKTOP:
-    if ((IS_STICKY(win)) || win->desknum == globals.desknum)
+    if (IS_STICKY(win) || win->desknum == globals.desknum)
       flag = 1;
     break;
 
+  case NO_SHOW_PAGE:
+    reverse = 1;
+    /* fall through to next case */
   case SHOW_PAGE:
     if (IS_STICKY(win)) {
       flag = 1;
     } else if (win->desknum == globals.desknum) {
       /* win and screen intersect if they are not disjoint in x and y */
-      flag = RECTANGLES_INTERSECT (win->x, win->y, win->width, win->height,
-				   0, 0, globals.screenx, globals.screeny);
+      flag = RECTANGLES_INTERSECT(
+	g.x, g.y, g.width, g.height,
+	manager->managed_g.x, manager->managed_g.y,
+	manager->managed_g.width, manager->managed_g.height);
     }
     break;
 
-  case NO_SHOW_DESKTOP:
-    /* Show the window if it's not on the current desk */
-    if (!(IS_STICKY(win)) && win->desknum != globals.desknum)
-      flag = 1;
+  case NO_SHOW_SCREEN:
+    reverse = 1;
+    /* fall through to next case */
+  case SHOW_SCREEN:
+    if (win->desknum == globals.desknum) {
+      /* win and screen intersect if they are not disjoint in x and y */
+      flag = RECTANGLES_INTERSECT(
+	g.x, g.y, g.width, g.height,
+	manager->managed_g.x, manager->managed_g.y,
+	manager->managed_g.width, manager->managed_g.height);
+    }
     break;
-
-  case NO_SHOW_PAGE:
-    /* Show the window if it's not in the viewport (on the page) */
-    if (!(IS_STICKY(win)) &&
-	 ( win->desknum != globals.desknum ||
-	   !(RECTANGLES_INTERSECT (win->x, win->y, win->width, win->height,
-      				   0, 0, globals.screenx, globals.screeny))))
-      flag = 1;
   }
+  flag ^= reverse;
+
   return flag;
 }
 
@@ -211,17 +235,22 @@ static void set_win_configuration (WinData *win, FvwmPacketBody *body)
   memcpy(&(win->flags), &(body->add_config_data.flags), sizeof(win->flags));
 }
 
-static void configure_colorsets (unsigned long *body)
+static void handle_config_info (unsigned long *body)
 {
   char *tline, *token;
   int color;
 
   tline = (char*)&(body[3]);
   token = PeekToken(tline, &tline);
-  if (StrEquals(token, "Colorset")) {
+  if (StrEquals(token, "Colorset"))
+  {
     color = LoadColorset(tline);
     change_colorset(color);
   }
+  else if (StrEquals(token, XINERAMA_CONFIG_STRING))
+  {
+    FScreenConfigureModule(tline);
+  }
 }
 
 static void configure_window (FvwmPacketBody *body)
@@ -363,22 +392,12 @@ static void new_window (FvwmPacketBody *body)
 {
   WinData *win;
 
-  win = new_windata();
+  win = id_to_win (body->add_config_data.w);
   memcpy(&(win->flags), &(body->add_config_data.flags), sizeof(win->flags));
-  if (!(IS_TRANSIENT(win)))
-  {
-    win->app_id = body->add_config_data.w;
-    win->app_id_set = 1;
-    set_win_configuration (win, body);
-
-    insert_win_hashtab (win);
-    check_win_complete (win);
-    check_in_window (win);
-  }
-  else
-  {
-    Free(win);
-  }
+  set_win_configuration (win, body);
+  got_configure (win->manager);
+  check_win_complete (win);
+  check_in_window (win);
 }
 
 static void destroy_window (FvwmPacketBody *body)
@@ -421,16 +440,32 @@ static void mini_icon (FvwmPacketBody *body)
 }
 #endif
 
-static void iconify (FvwmPacketBody *body, int dir)
+static void icon_location(FvwmPacketBody *body)
 {
   Ulong app_id = body->minimal_data.app_id;
   WinData *win;
 
-  win = id_to_win (app_id);
+  win = id_to_win(app_id);
+  if (win == NULL)
+  {
+    return;
+  }
+  win->icon_g.x = body->icon_data.xpos;
+  win->icon_g.y = body->icon_data.ypos;
+  win->icon_g.width = body->icon_data.icon_width;
+  win->icon_g.height = body->icon_data.icon_height;
+  check_in_window (win);
+}
 
-  set_win_iconified (win, dir);
+static void iconify(FvwmPacketBody *body, int dir)
+{
+  Ulong app_id = body->minimal_data.app_id;
+  WinData *win;
 
-  check_win_complete (win);
+  win = id_to_win(app_id);
+  set_win_iconified(win, dir);
+  icon_location(body);
+  check_win_complete(win);
   check_in_window (win);
 }
 
@@ -470,7 +505,7 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
   {
   case M_CONFIG_INFO:
     ConsoleDebug (FVWM, "DEBUG::M_CONFIG_INFO\n");
-    configure_colorsets ((unsigned long*)body);
+    handle_config_info ((unsigned long*)body);
     break;
 
   case M_CONFIGURE_WINDOW:
@@ -536,6 +571,10 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
     SendUnlockNotification(Fvwm_fd);
     break;
 
+  case M_ICON_LOCATION:
+    icon_location(body);
+    break;
+
   case M_END_WINDOWLIST:
     ConsoleDebug (FVWM, "DEBUG::M_END_WINDOWLIST\n");
     ConsoleDebug (FVWM,
@@ -597,17 +636,19 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
 
 void ReadFvwmPipe (void)
 {
-    FvwmPacket* packet;
+  FvwmPacket* packet;
 
-    PrintMemuse();
-    ConsoleDebug(FVWM, "DEBUG: entering ReadFvwmPipe\n");
+  PrintMemuse();
+  ConsoleDebug(FVWM, "DEBUG: entering ReadFvwmPipe\n");
 
-    if ( (packet = ReadFvwmPacket(Fvwm_fd[1])) == NULL )
-    {
-	exit(0);
-    }
-    else
-	ProcessMessage( packet->type, (FvwmPacketBody*) packet->body );
+  if ( (packet = ReadFvwmPacket(Fvwm_fd[1])) == NULL )
+  {
+    exit(0);
+  }
+  else
+  {
+    ProcessMessage( packet->type, (FvwmPacketBody*) packet->body );
+  }
 
-    ConsoleDebug(FVWM, "DEBUG: leaving ReadFvwmPipe\n");
+  ConsoleDebug(FVWM, "DEBUG: leaving ReadFvwmPipe\n");
 }
