@@ -19,19 +19,18 @@
 #include "x.h"
 #include "xmanager.h"
 #include "libs/fvwmlib.h"
+#include "libs/FScreen.h"
+#include "libs/FShape.h"
 
 static char const rcsid[] =
-  "$Id: x.c,v 1.44 2000/12/03 15:56:03 domivogt Exp $";
+  "$Id: x.c,v 1.54 2001/09/23 07:03:58 olicha Exp $";
 
 #define GRAB_EVENTS (ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|EnterWindowMask|LeaveWindowMask)
 
-#ifdef SHAPE
-static int shapeEventBase, shapeErrorBase;
-#endif
-
 Display *theDisplay;
 Window theRoot;
 int theScreen;
+static Atom _XA_WM_DEL_WIN;
 extern char *MyName;
 
 static enum {
@@ -54,6 +53,7 @@ static void grab_pointer (WinManager *man)
     }
     grab_state = HAVE_GRABBED;
   }
+  return;
 }
 
 static int lookup_color (char *name, Pixel *ans)
@@ -301,24 +301,12 @@ void xevent_loop (void)
       saveEvent = theEvent;
       /* eat up all but last ConfigureNotify events */
       while (XPending(theDisplay) &&
-	     XCheckMaskEvent(theDisplay, ConfigureNotify, &theEvent))
+	     XCheckTypedEvent(theDisplay, ConfigureNotify, &theEvent))
       {
-	if (theEvent.type != ConfigureNotify)
-	{
-	  /* I can't fathom how this could ever happen.  I wouldn't believe it
-	   * if I hadn't seen it with my own eyes: instead of a ConfigureNotify
-	   * I sometimes get an EnterNotify event instead. Sounds like a broken
-	   * X server. */
-	  XPutBackEvent(theDisplay, &theEvent);
-	  break;
-	}
-	else
-	{
-	  saveEvent = theEvent;
-	  /* check for movement on all events */
-	  if (theEvent.xconfigure.send_event)
-	    moved = True;
-	}
+	saveEvent = theEvent;
+	/* check for movement on all events */
+	if (theEvent.xconfigure.send_event)
+	  moved = True;
       }
       theEvent = saveEvent;
 
@@ -368,7 +356,10 @@ void xevent_loop (void)
       set_manager_width (man, theEvent.xconfigure.width);
       ConsoleDebug (X11, "\tboxwidth = %d\n", man->geometry.boxwidth);
       if (force_redraw)
+      {
 	force_manager_redraw(man);
+	force_redraw = 0;
+      }
       else
 	draw_manager (man);
 
@@ -387,6 +378,9 @@ void xevent_loop (void)
 
     case MapNotify:
       ConsoleDebug (X11, "XEVENT: MapNotify\n");
+      if (globals.transient && !grab_state != HAVE_GRABBED) {
+	grab_pointer (man);
+      }
       force_manager_redraw (man);
       break;
 
@@ -394,21 +388,27 @@ void xevent_loop (void)
       ConsoleDebug (X11, "XEVENT: UnmapNotify\n");
       break;
 
+    case ClientMessage:
+      if (theEvent.xclient.format == 32 &&
+	  theEvent.xclient.data.l[0]==_XA_WM_DEL_WIN)
+      {
+	DeadPipe(1);
+      }
+      break;
+
     case DestroyNotify:
       ConsoleDebug(X11, "XEVENT: DestroyNotify\n");
       DeadPipe(0);
       break;
 
     default:
-#ifdef SHAPE
-      if (theEvent.type == shapeEventBase + ShapeNotify) {
-	XShapeEvent *xev = (XShapeEvent *)&theEvent;
+      if (FShapesSupported && theEvent.type == FShapeEventBase + FShapeNotify) {
+	FShapeEvent *xev = (FShapeEvent *)&theEvent;
 	ConsoleDebug (X11, "XEVENT: ShapeNotify\n");
 	ConsoleDebug (X11, "\tx, y, w, h = %d, %d, %d, %d\n",
 		      xev->x, xev->y, xev->width, xev->height);
 	break;
       }
-#endif
       ConsoleDebug (X11, "XEVENT: unknown\n");
       break;
     }
@@ -479,7 +479,7 @@ void map_manager (WinManager *man)
     set_manager_window_mapping (man, 1);
     XFlush (theDisplay);
     if (globals.transient) {
-      /* wait for an expose event to actually do the grab */
+      /* wait for a map event to actually do the grab */
       grab_state = NEED_TO_GRAB;
     }
   }
@@ -511,8 +511,6 @@ void X_init_manager (int man_id)
 
   man->geometry.cols = DEFAULT_NUM_COLS;
   man->geometry.rows = DEFAULT_NUM_ROWS;
-  man->geometry.x = 0;
-  man->geometry.y = 0;
   man->gravity = NorthWestGravity;
 
   man->select_button = NULL;
@@ -638,11 +636,24 @@ void X_init_manager (int man_id)
         man->geometry.boxheight = height;
     }
   }
+  FScreenGetScrRect(NULL, FSCREEN_PRIMARY, &man->managed_g.x, &man->managed_g.y,
+		    &man->managed_g.width, &man->managed_g.height);
+  man->geometry.x = man->managed_g.x;
+  man->geometry.y = man->managed_g.y;
   if (man->geometry_str) {
-    geometry_mask = XParseGeometry (man->geometry_str, &man->geometry.x,
-				    &man->geometry.y, &man->geometry.cols,
-				    &man->geometry.rows);
+    int scr;
 
+    geometry_mask = FScreenParseGeometryWithScreen(
+      man->geometry_str, &man->geometry.x, &man->geometry.y,
+      &man->geometry.cols, &man->geometry.rows, &scr);
+    FScreenGetScrRect(
+      NULL, scr, &man->managed_g.x, &man->managed_g.y,
+      &man->managed_g.width, &man->managed_g.height);
+
+    if (geometry_mask & XValue)
+      man->geometry.x += man->managed_g.x;
+    if (geometry_mask & YValue)
+	man->geometry.y += man->managed_g.y;
     if ((geometry_mask & XValue) || (geometry_mask & YValue)) {
       man->sizehints_flags |= USPosition;
       if (geometry_mask & XNegative)
@@ -678,20 +689,24 @@ void X_init_manager (int man_id)
 
   man->geometry.width  = man->geometry.cols * man->geometry.boxwidth;
   man->geometry.height = man->geometry.rows * man->geometry.boxheight;
-
   if ((geometry_mask & XValue) && (geometry_mask & XNegative))
-    man->geometry.x += globals.screenx - man->geometry.width;
+    man->geometry.x += man->managed_g.width - man->geometry.width;
   if ((geometry_mask & YValue) && (geometry_mask & YNegative))
-    man->geometry.y += globals.screeny - man->geometry.height;
-
+    man->geometry.y += man->managed_g.height - man->geometry.height;
   if (globals.transient) {
     Window dummyroot, dummychild;
     int junk;
     unsigned int ujunk;
+    fscreen_scr_arg fscr;
 
     XQueryPointer(theDisplay, theRoot, &dummyroot, &dummychild,
-		  &man->geometry.x,
-		  &man->geometry.y, &junk, &junk, &ujunk);
+		  &man->geometry.x, &man->geometry.y, &junk, &junk, &ujunk);
+    fscr.xypos.x = man->geometry.x;
+    fscr.xypos.y = man->geometry.y;
+    FScreenGetScrRect(
+      &fscr, FSCREEN_XYPOS,
+      &man->managed_g.x, &man->managed_g.y,
+      &man->managed_g.width, &man->managed_g.height);
     man->geometry.dir |= GROW_DOWN | GROW_RIGHT;
     man->sizehints_flags |= USPosition;
   }
@@ -721,6 +736,8 @@ void X_init_manager (int man_id)
     man->geometry.dir |= GROW_DOWN | GROW_RIGHT;
     man->gravity = NorthWestGravity;
   }
+
+  return;
 }
 
 void create_manager_window (int man_id)
@@ -753,11 +770,11 @@ void create_manager_window (int man_id)
   sizehints.height = man->geometry.height;
   sizehints.base_width = sizehints.min_width = 0;
   sizehints.base_height = 0;
-  sizehints.max_width = globals.screenx;
+  sizehints.max_width = man->managed_g.width;
   if (man->geometry.dir & GROW_FIXED)
   {
     sizehints.min_height = man->geometry.boxheight;
-    sizehints.max_height = globals.screeny;
+    sizehints.max_height = man->managed_g.height;
   }
   else
   {
@@ -773,7 +790,6 @@ void create_manager_window (int man_id)
   sizehints.x = man->geometry.x;
   sizehints.y = man->geometry.y;
 
-
   ConsoleDebug (X11, "hints: x, y, w, h = %d %d %d %d)\n",
 		sizehints.x, sizehints.y,
 		sizehints.base_width, sizehints.base_height);
@@ -794,14 +810,36 @@ void create_manager_window (int man_id)
   else
     winattr.event_mask |= ButtonPressMask;
 
+  if (man->res == SHOW_SCREEN || man->res == NO_SHOW_SCREEN)
+  {
+    fscreen_scr_arg fscr;
+
+    fscr.xypos.x = sizehints.x;
+    fscr.xypos.y = sizehints.y;
+    FScreenGetScrRect(
+      &fscr, FSCREEN_XYPOS,
+      &man->managed_g.x, &man->managed_g.y,
+      &man->managed_g.width, &man->managed_g.height);
+  }
+  else
+  {
+    FScreenGetScrRect(
+      NULL, FSCREEN_GLOBAL,
+      &man->managed_g.x, &man->managed_g.y,
+      &man->managed_g.width, &man->managed_g.height);
+  }
+
   man->theWindow = XCreateWindow(theDisplay, theRoot, sizehints.x, sizehints.y,
 				 man->geometry.width, man->geometry.height,
 				 0, Pdepth, InputOutput, Pvisual, winattrmask,
 				 &winattr);
   XSetWMNormalHints(theDisplay, man->theWindow, &sizehints);
-#ifdef SHAPE
-  XShapeSelectInput (theDisplay, man->theWindow, ShapeNotifyMask);
-#endif
+  FShapeSelectInput (theDisplay, man->theWindow, FShapeNotifyMask);
+  if (globals.transient)
+  {
+    XSetTransientForHint(theDisplay, man->theWindow, theRoot);
+  }
+  XSetWMProtocols(theDisplay, man->theWindow, &_XA_WM_DEL_WIN, 1);
 
   /* We really want the bit gravity to be NorthWest, so that can minimize
      redraws. But, we had to have an appropriate bit gravity when creating
@@ -900,24 +938,20 @@ void init_display (void)
     ShutMeDown (1);
   }
   XSetErrorHandler (handle_error);
+  _XA_WM_DEL_WIN = XInternAtom(theDisplay, "WM_DELETE_WINDOW", 0);
   InitPictureCMap (theDisplay);
+  FScreenInit(theDisplay);
   AllocColorset(0);
+  FShapeInit(theDisplay);
   x_fd = XConnectionNumber (theDisplay);
   theScreen = DefaultScreen (theDisplay);
   theRoot = RootWindow (theDisplay, theScreen);
 #ifdef TEST_MONO
   Pdepth = 2;
 #endif
-  globals.screenx = DisplayWidth (theDisplay, theScreen);
-  globals.screeny = DisplayHeight (theDisplay, theScreen);
 
-#ifdef SHAPE
-  globals.shapes_supported = XShapeQueryExtension (theDisplay, &shapeEventBase,
-						   &shapeErrorBase);
-#endif
-
-  ConsoleDebug (X11, "screen width: %ld\n", globals.screenx);
-  ConsoleDebug (X11, "screen height: %ld\n", globals.screeny);
+  ConsoleDebug (X11, "screen width: %d\n", globals.screen_g.width);
+  ConsoleDebug (X11, "screen height: %d\n", globals.screen_g.height);
 }
 
 void change_colorset(int color)
