@@ -47,6 +47,8 @@
 
 #include "libs/Module.h"
 #include "libs/fvwmlib.h"
+#include "libs/FScreen.h"
+#include "libs/FShape.h"
 #include "libs/Colorset.h"
 #ifdef DEBUG
 #  define FVWM_DEBUG_MSGS   /* Do we need this? */
@@ -121,13 +123,12 @@ int window_w=0, window_h=0, window_x=0, window_y=0;
 int icon_x=-10000, icon_y=-10000, icon_w=0, icon_h=0;
 int usposition = 0,uselabel = 1;
 int xneg = 0, yneg = 0;
+int icon_xneg = 0, icon_yneg = 0;
 extern DeskInfo *Desks;
 int StartIconic = 0;
 int MiniIcons = 0;
 int LabelsBelow = 0;
-#ifdef SHAPE
 int ShapeLabels = 0;
-#endif
 int Rows = -1, Columns = -1;
 int desk1=0, desk2 =0;
 int ndesks = 0;
@@ -233,23 +234,33 @@ int main(int argc, char **argv)
   fd_width = GetFdWidth();
 
   opt_num = 6;
-  if (argc >= 7 && StrEquals(argv[opt_num], "-transient"))
-    {
-      opt_num++;
-      is_transient = True;
+  if (argc >= 7 && (StrEquals(argv[opt_num], "-transient") ||
+		    StrEquals(argv[opt_num], "transient")))
+  {
+    opt_num++;
+    is_transient = True;
       do_ignore_next_button_release = True;
-    }
+  }
 
   /* Check for an alias */
   if (argc >= opt_num + 1)
     {
-      if (strspn(argv[opt_num], "0123456789") != strlen(argv[opt_num]) &&
-	  !StrEquals(argv[opt_num], "*"))
+      char *s;
+
+      if (!StrEquals(argv[opt_num], "*"))
+      {
+	for (s = argv[opt_num]; *s; s++)
 	{
-	  free(MyName);
-	  MyName=strdup(argv[opt_num]);
-	  opt_num++;
+	  if (!isdigit(*s) &&
+	      (*s != '-' || s != argv[opt_num] || *(s+1) == 0))
+	  {
+	    free(MyName);
+	    MyName=safestrdup(argv[opt_num]);
+	    opt_num++;
+	    break;
+	  }
 	}
+      }
     }
 
   if (argc < opt_num + 1)
@@ -280,12 +291,11 @@ int main(int argc, char **argv)
   ndesks = desk2 - desk1 + 1;
 
   Desks = (DeskInfo *)safemalloc(ndesks*sizeof(DeskInfo));
+  memset(Desks, 0, ndesks * sizeof(DeskInfo));
   for(i=0;i<ndesks;i++)
     {
       sprintf(line,"Desk %d",i+desk1);
       CopyString(&Desks[i].label,line);
-      Desks[i].Dcolor = NULL;
-      Desks[i].bgPixmap = NULL;
       Desks[i].colorset = -1;
       Desks[i].highcolorset = -1;
       Desks[i].ballooncolorset = -1;
@@ -300,7 +310,9 @@ int main(int argc, char **argv)
     }
   x_fd = XConnectionNumber(dpy);
   InitPictureCMap(dpy);
+  FScreenInit(dpy);
   AllocColorset(0);
+  FShapeInit(dpy);
 
   Scr.screen = DefaultScreen(dpy);
   Scr.Root = RootWindow(dpy, Scr.screen);
@@ -350,34 +362,34 @@ int main(int argc, char **argv)
 #endif
 
   if (PagerFore == NULL)
-    PagerFore = strdup("black");
+    PagerFore = safestrdup("black");
 
   if (PagerBack == NULL)
-    PagerBack = strdup("white");
+    PagerBack = safestrdup("white");
 
   if (HilightC == NULL)
-    HilightC = strdup(PagerFore);
+    HilightC = safestrdup(PagerFore);
 
   if (WindowLabelFormat == NULL)
-    WindowLabelFormat = strdup("%i");
+    WindowLabelFormat = safestrdup("%i");
 
   if (font_string == NULL)
-    font_string = strdup("fixed");
+    font_string = safestrdup("fixed");
 
   if ((HilightC == NULL) && (HilightPixmap == NULL))
     HilightDesks = 0;
 
   if (BalloonFont == NULL)
-    BalloonFont = strdup("fixed");
+    BalloonFont = safestrdup("fixed");
 
   if (BalloonBorderColor == NULL)
-    BalloonBorderColor = strdup("black");
+    BalloonBorderColor = safestrdup("black");
 
   if (BalloonTypeString == NULL)
-    BalloonTypeString = strdup("%i");
+    BalloonTypeString = safestrdup("%i");
 
   if (BalloonFormatString == NULL)
-    BalloonFormatString = strdup("%i");
+    BalloonFormatString = safestrdup("%i");
 
   /* open a pager window */
   initialize_pager();
@@ -394,24 +406,43 @@ int main(int argc, char **argv)
 #endif
 
   if (is_transient)
+  {
+    Bool is_pointer_grabbed = False;
+    Bool is_keyboard_grabbed = False;
+    XSync(dpy,0);
+    for (i = 0; i < 50 && !(is_pointer_grabbed && is_keyboard_grabbed); i++)
     {
-      XSync(dpy,0);
-      while (i < 1000 &&
-	     XGrabPointer(dpy, Scr.Root, True,
-			  ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|
-			  PointerMotionMask|EnterWindowMask|LeaveWindowMask,
-			  GrabModeAsync, GrabModeAsync, None,
-			  None, CurrentTime) != GrabSuccess)
-	{
-	  i++;
-	  /* If you go too fast, other windows may not get a change to release
-	   * any grab that they have. */
-	  usleep(1000);
-	}
-      XGrabKeyboard(dpy, Scr.Root, True, GrabModeAsync,
-		    GrabModeAsync, CurrentTime);
-      XSync(dpy,0);
+      if (!is_pointer_grabbed &&
+	  XGrabPointer(
+	    dpy, Scr.Root, True,
+	    ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|
+	    PointerMotionMask|EnterWindowMask|LeaveWindowMask, GrabModeAsync,
+	    GrabModeAsync, None, None, CurrentTime) == GrabSuccess)
+      {
+	is_pointer_grabbed = True;
+      }
+      if (!is_keyboard_grabbed &&
+	  XGrabKeyboard(
+	    dpy, Scr.Root, True, GrabModeAsync, GrabModeAsync, CurrentTime) ==
+	  GrabSuccess)
+      {
+	is_keyboard_grabbed = True;
+      }
+      /* If you go too fast, other windows may not get a change to release
+       * any grab that they have. */
+      usleep(20000);
     }
+    if (!is_pointer_grabbed)
+    {
+      XBell(dpy, 0);
+      fprintf(stderr,
+	      "%s: could not grab pointer in transient mode. exiting.\n",
+	      MyName);
+      exit(1);
+    }
+
+    XSync(dpy,0);
+  }
 
   /* tell fvwm we're running */
   SendFinishedStartupNotification(fd);
@@ -503,7 +534,7 @@ void process_message( FvwmPacket* packet )
       list_iconify(body);
       break;
     case M_DEICONIFY:
-      list_deiconify(body);
+      list_deiconify(body, length);
       break;
     case M_RES_CLASS:
     case M_RES_NAME:
@@ -523,7 +554,7 @@ void process_message( FvwmPacket* packet )
       list_restack(body,length);
       break;
     case M_CONFIG_INFO:
-      list_colorset(body);
+      list_config_info(body);
       break;
     default:
       /* ignore unknown packet */
@@ -576,18 +607,7 @@ void list_add(unsigned long *body)
       i++;
     }
   *prev = (PagerWindow *)safemalloc(sizeof(PagerWindow));
-  (*prev)->next = NULL;
-  (*prev)->pager_view_width = 0;
-  (*prev)->pager_view_height = 0;
-  (*prev)->icon_view_width = 0;
-  (*prev)->icon_view_height = 0;
-  (*prev)->icon_name = NULL;
-  (*prev)->window_name = NULL;
-  (*prev)->res_name = NULL;
-  (*prev)->res_class = NULL;
-  (*prev)->window_label = NULL;
-  (*prev)->mini_icon.picture = 0;
-
+  memset(*prev, 0, sizeof(PagerWindow));
   (*prev)->w = cfgpacket->w;
   (*prev)->frame = cfgpacket->frame;
   (*prev)->t = (char *) cfgpacket->fvwmwin;
@@ -604,13 +624,13 @@ void list_add(unsigned long *body)
 
   if (win_pix_set)
   {
-        (*prev)->text = win_fore_pix;
-        (*prev)->back = win_back_pix;
+    (*prev)->text = win_fore_pix;
+    (*prev)->back = win_back_pix;
   }
   else
   {
-        (*prev)->text = body[22];
-        (*prev)->back = body[23];
+    (*prev)->text = cfgpacket->TextPixel;
+    (*prev)->back = cfgpacket->BackPixel;
   }
   AddNewWindow(*prev);
 }
@@ -658,8 +678,8 @@ void list_configure(unsigned long *body)
   }
   else
   {
-    t->text = body[22];
-    t->back = body[23];
+    t->text = cfgpacket->TextPixel;
+    t->back = cfgpacket->BackPixel;
   }
   if(IS_ICONIFIED(t))
   {
@@ -685,7 +705,7 @@ void list_configure(unsigned long *body)
     ChangeDeskForWindow(t, cfgpacket->desk);
   }
   else
-    MoveResizePagerView(t);
+    MoveResizePagerView(t, False);
 }
 
 /***********************************************************************
@@ -717,7 +737,14 @@ void list_destroy(unsigned long *body)
       XDestroyWindow(dpy,t->IconView);
       if(FocusWin == t)
 	FocusWin = NULL;
-
+      if(t->res_class != NULL)
+	free(t->res_class);
+      if(t->res_name != NULL)
+	free(t->res_name);
+      if(t->window_name != NULL)
+	free(t->window_name);
+      if(t->icon_name != NULL)
+	free(t->icon_name);
       free(t);
     }
 }
@@ -733,15 +760,24 @@ void list_focus(unsigned long *body)
   PagerWindow *t;
   Window target_w;
   extern Pixel focus_pix, focus_fore_pix;
-  target_w = body[0];
+  Bool do_force_update = False;
 
+  target_w = body[0];
   if (win_hi_pix_set)
   {
+    if (focus_pix != win_hi_back_pix || focus_fore_pix != win_hi_fore_pix)
+    {
+      do_force_update = True;
+    }
     focus_pix = win_hi_back_pix;
     focus_fore_pix = win_hi_fore_pix;
   }
   else
   {
+    if (focus_pix != body[4] || focus_fore_pix != body[3])
+    {
+      do_force_update = True;
+    }
     focus_pix = body[4];
     focus_fore_pix = body[3];
   }
@@ -750,12 +786,12 @@ void list_focus(unsigned long *body)
   {
     t = t->next;
   }
-  if(t != FocusWin)
+  if (t != FocusWin || do_force_update)
   {
-    if(FocusWin != NULL)
+    if (FocusWin != NULL)
       Hilight(FocusWin, False);
     FocusWin = t;
-    if(FocusWin != NULL)
+    if (FocusWin != NULL)
       Hilight(FocusWin, True);
   }
 }
@@ -786,7 +822,7 @@ void list_new_page(unsigned long *body)
     Scr.VHeight = Scr.VyMax + Scr.MyDisplayHeight;
     ReConfigure();
   }
-  MovePage();
+  MovePage(False);
   MoveStickyWindows();
   Hilight(FocusWin,True);
 }
@@ -926,8 +962,27 @@ void list_new_desk(unsigned long *body)
     XClearWindow(dpy, Desks[0].w);
     XClearWindow(dpy, Desks[0].title_w);
   } /* if (fAlwaysCurrentDesk && oldDesk != Scr.CurrentDesk) */
+  else if (!fAlwaysCurrentDesk)
+  {
+    int i;
+    char *name;
+    char line[100];
 
-  MovePage();
+    i = Scr.CurrentDesk - desk1;
+    if (i >= 0 && i < ndesks && Desks[i].label != NULL)
+    {
+      name = Desks[i].label;
+    }
+    else
+    {
+      sprintf(line, "Desk %d", Scr.CurrentDesk);
+      name = &(line[0]);
+    }
+      XStoreName(dpy, Scr.Pager_w, name);
+      XSetIconName(dpy, Scr.Pager_w, name);
+  }
+
+  MovePage(True);
   DrawGrid(oldDesk - desk1,1);
   DrawGrid(Scr.CurrentDesk - desk1,1);
   MoveStickyWindows();
@@ -1038,7 +1093,7 @@ void list_iconify(unsigned long *body)
       if ( t->w == Scr.Pager_w )
 	ShowBalloons = ShowIconBalloons;
 
-      MoveResizePagerView(t);
+      MoveResizePagerView(t, True);
     }
 }
 
@@ -1050,7 +1105,7 @@ void list_iconify(unsigned long *body)
  *
  ***********************************************************************/
 
-void list_deiconify(unsigned long *body)
+void list_deiconify(unsigned long *body, unsigned long length)
 {
   PagerWindow *t;
   Window target_w;
@@ -1058,31 +1113,36 @@ void list_deiconify(unsigned long *body)
   target_w = body[0];
   t = Start;
   while((t!= NULL)&&(t->w != target_w))
-    {
-      t = t->next;
-    }
-  if(t== NULL)
-    {
-      return;
-    }
+  {
+    t = t->next;
+  }
+  if (t == NULL)
+  {
+    return;
+  }
   else
+  {
+    SET_ICONIFIED(t, False);
+    if (length >= 11 + FvwmPacketHeaderSize)
     {
-      SET_ICONIFIED(t, False);
       t->frame_x = body[7];
       t->frame_y = body[8];
       t->frame_width = body[9];
       t->frame_height = body[10];
-      t->x = body[7];
-      t->y = body[8];
-      t->width = body[9];
-      t->height = body[10];
-
-      /* if deiconifying main pager window turn balloons on or off */
-      if ( t->w == Scr.Pager_w )
-	ShowBalloons = ShowPagerBalloons;
-
-      MoveResizePagerView(t);
     }
+    t->x = t->frame_x;
+    t->y = t->frame_y;
+    t->width = t->frame_width;
+    t->height = t->frame_height;
+
+    /* if deiconifying main pager window turn balloons on or off */
+    if ( t->w == Scr.Pager_w )
+      ShowBalloons = ShowPagerBalloons;
+
+    MoveResizePagerView(t, True);
+  }
+
+  return;
 }
 
 
@@ -1210,43 +1270,43 @@ void list_restack(unsigned long *body, unsigned long length)
   wins = (Window *) safemalloc (length * sizeof (Window));
   /* first restack in the icon view */
   j = 0;
-  for (i = 0; i < (length - 4); i+=3)
+  for (i = 0; i < (length - FvwmPacketHeaderSize); i += 3)
   {
     target_w = body[i];
     t = Start;
     while((t!= NULL)&&(t->w != target_w))
-     {
-       t = t->next;
-     }
+    {
+      t = t->next;
+    }
     if (t != NULL)
-     {
-       wins[j++] = t->IconView;
-     }
-   }
-   XRestackWindows(dpy, wins, j);
+    {
+      wins[j++] = t->IconView;
+    }
+  }
+  XRestackWindows(dpy, wins, j);
 
   /* now restack each desk separately, since they have separate roots */
   for (d = 0; d < ndesks; d++)
-   {
-     j = 0;
-     for (i = 0; i < (length - 4); i+=3)
+  {
+    j = 0;
+    for (i = 0; i < (length - 4); i+=3)
+    {
+      target_w = body[i];
+      t = Start;
+      while((t!= NULL)&&((t->w != target_w)||(t->desk != d+desk1)))
       {
-        target_w = body[i];
-        t = Start;
-        while((t!= NULL)&&((t->w != target_w)||(t->desk != d+desk1)))
-         {
-            t = t->next;
-         }
-        if (t != NULL)
-         {
-           if (t->PagerView != None)
-            {
-             wins[j++] = t->PagerView;
-            }
-         }
-       }
-      XRestackWindows(dpy, wins, j);
+	t = t->next;
+      }
+      if (t != NULL)
+      {
+	if (t->PagerView != None)
+	{
+	  wins[j++] = t->PagerView;
+	}
+      }
     }
+    XRestackWindows(dpy, wins, j);
+  }
   free (wins);
 }
 
@@ -1289,17 +1349,22 @@ void list_end(void)
 }
 
 
-void list_colorset(unsigned long *body)
+void list_config_info(unsigned long *body)
 {
   char *tline, *token;
   int color;
 
   tline = (char*)&(body[3]);
   token = PeekToken(tline, &tline);
-  if (StrEquals(token, "Colorset")) {
+  if (StrEquals(token, "Colorset"))
+  {
     color = LoadColorset(tline);
     change_colorset(color);
   }
+  else if (StrEquals(token, XINERAMA_CONFIG_STRING))
+  {
+    FScreenConfigureModule(tline);
+  }
 }
 
 
@@ -1314,10 +1379,10 @@ int My_XNextEvent(Display *dpy, XEvent *event)
   static int miss_counter = 0;
 
   if(XPending(dpy))
-    {
-      XNextEvent(dpy,event);
-      return 1;
-    }
+  {
+    XNextEvent(dpy,event);
+    return 1;
+  }
 
   FD_ZERO(&in_fdset);
   FD_SET(x_fd,&in_fdset);
@@ -1456,11 +1521,15 @@ void ParseOptions(void)
     resource_string = arg1 = arg2 = NULL;
 
     token = PeekToken(tline, &next);
-    if(StrEquals(token, "Colorset"))
+    if (StrEquals(token, "Colorset"))
     {
       LoadColorset(next);
       continue;
     }
+    else if (StrEquals(token, XINERAMA_CONFIG_STRING))
+    {
+      FScreenConfigureModule(next);
+    }
     else if (StrEquals(token, "DesktopSize"))
     {
       token = PeekToken(next, &next);
@@ -1542,7 +1611,14 @@ void ParseOptions(void)
     }
     else if (StrEquals(resource, "Geometry"))
     {
-      flags = XParseGeometry(arg1,&g_x,&g_y,&width,&height);
+      window_w = 0;
+      window_h = 0;
+      window_x = 0;
+      window_y = 0;
+      xneg = 0;
+      yneg = 0;
+      usposition = 0;
+      flags = FScreenParseGeometry(arg1,&g_x,&g_y,&width,&height);
       if (flags & WidthValue)
       {
 	window_w = width;
@@ -1572,7 +1648,13 @@ void ParseOptions(void)
     }
     else if (StrEquals(resource, "IconGeometry"))
     {
-      flags = XParseGeometry(arg1,&g_x,&g_y,&width,&height);
+      icon_w = 0;
+      icon_h = 0;
+      icon_x = -10000;
+      icon_y = -10000;
+      icon_xneg = 0;
+      icon_yneg = 0;
+      flags = FScreenParseGeometry(arg1,&g_x,&g_y,&width,&height);
       if (flags & WidthValue)
 	icon_w = width;
       if (flags & HeightValue)
@@ -1580,10 +1662,18 @@ void ParseOptions(void)
       if (flags & XValue)
       {
 	icon_x = g_x;
+        if (flags & XNegative)
+        {
+          icon_xneg = 1;
+        }
       }
       if (flags & YValue)
       {
 	icon_y = g_y;
+        if (flags & YNegative)
+        {
+          icon_yneg = 1;
+        }
       }
     }
     else if (StrEquals(resource, "Label"))
@@ -1634,11 +1724,22 @@ void ParseOptions(void)
     {
       if (font_string)
 	free(font_string);
-      CopyString(&font_string,next);
-      if (font_string[0] == '\"')
-        font_string++;
-      if (font_string[strlen(font_string)-1] == '\"' )
-        font_string[strlen(font_string)-1] = 0;
+      if (next && next[0] == '\"')
+      {
+	int l;
+
+	next++;
+	CopyString(&font_string, next);
+	l = strlen(font_string);
+	if (l > 0 && font_string[l - 1] == '\"')
+	{
+	  font_string[l - 1] = 0;
+	}
+      }
+      else
+      {
+	CopyString(&font_string,next);
+      }
       if(strncasecmp(font_string,"none",4) == 0)
 	uselabel = 0;
     }
@@ -1833,8 +1934,23 @@ void ParseOptions(void)
     {
       if (smallFont)
 	free(smallFont);
-      CopyString(&smallFont,next);
-      if(strncasecmp(smallFont,"none",4) == 0)
+      if (next && next[0] == '\"')
+      {
+	int l;
+
+	next++;
+	CopyString(&smallFont, next);
+	l = strlen(smallFont);
+	if (l > 0 && smallFont[l - 1] == '\"')
+	{
+	  smallFont[l - 1] = 0;
+	}
+      }
+      else
+      {
+	CopyString(&smallFont,next);
+      }
+      if (strncasecmp(smallFont,"none",4) == 0)
       {
 	free(smallFont);
 	smallFont = NULL;
@@ -1860,16 +1976,14 @@ void ParseOptions(void)
     {
       LabelsBelow = 0;
     }
-#ifdef SHAPE
-    else if (StrEquals(resource, "ShapeLabels"))
+    else if (FHaveShapeExtension && StrEquals(resource, "ShapeLabels"))
     {
       ShapeLabels = 1;
     }
-    else if (StrEquals(resource, "NoShapeLabels"))
+    else if (FHaveShapeExtension && StrEquals(resource, "NoShapeLabels"))
     {
       ShapeLabels = 0;
     }
-#endif
     else if (StrEquals(resource, "Rows"))
     {
       sscanf(arg1,"%d",&Rows);
@@ -1995,7 +2109,7 @@ void ParseOptions(void)
     {
       if (BalloonFont)
 	free(BalloonFont);
-      CopyString(&BalloonFont, next);
+      CopyStringWithQuotes(&BalloonFont, next);
     }
 
     else if (StrEquals(resource, "BalloonBorderColor"))
@@ -2063,15 +2177,12 @@ PagerStringList *NewPagerStringItem(PagerStringList *last, int desk)
   PagerStringList *newitem;
 
   newitem = (PagerStringList *)safemalloc(sizeof(PagerStringList));
+  memset(newitem, 0, sizeof(PagerStringList));
   last->next = newitem;
   newitem->colorset = -1;
   newitem->highcolorset = -1;
   newitem->ballooncolorset = -1;
   newitem->desk = desk;
-  newitem->next = NULL;
-  newitem->label = NULL;
-  newitem->Dcolor = NULL;
-  newitem->bgPixmap = NULL;
 
   return newitem;
 }
