@@ -162,9 +162,12 @@ int  win_width    = 5,
 			 *  0 hidden -> not hidden (for the events loop) */
      FocusInWin = 0;    /* 1 if the Taskbar has the focus */
 
+static int auto_stick_y = 0;
+
 static volatile sig_atomic_t AlarmSet = NOT_SET;
 static volatile sig_atomic_t tip_window_alarm = False;
 static volatile sig_atomic_t hide_taskbar_alarm = False;
+static volatile sig_atomic_t stick_taskbar_alarm = False;
 
 
 int UpdateInterval = 30;
@@ -237,11 +240,12 @@ static void ParseConfigLine(char *tline);
 static void ShutMeDown(void);
 static RETSIGTYPE TerminateHandler(int sig);
 static RETSIGTYPE Alarm(int sig);
-static void SetAlarm(int event);
-static void ClearAlarm(void);
+static void SetAlarm(tb_alarm_t event);
+static void ClearAlarm(tb_alarm_t event);
 static void DoAlarmAction(void);
 static int ErrorHandler(Display*, XErrorEvent*);
 static Bool change_colorset(int cset, Bool force);
+static void WarpTaskBar(int y);
 int IsItemIndexIconSuppressed(List *list, int i);
 
 /******************************************************************************
@@ -416,9 +420,10 @@ void EndLessLoop(void)
         LoopOnEvents();
 
       if (FD_ISSET(Fvwm_fd[1], &readset))
-        ReadFvwmPipe();
+      {
+	ReadFvwmPipe();
+      }
     }
-
     DoAlarmAction();
 
     DrawGoodies();
@@ -548,7 +553,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 
 	XSetWMNormalHints(dpy,win,&hints);
 
- 	if (AutoStick)
+	if (AutoStick)
 	  XMoveResizeWindow(dpy, win, win_x, win_y, win_width, win_height);
 	else
 	  XResizeWindow(dpy, win, win_width, win_height);
@@ -687,7 +692,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	redraw = 1;
       }
     }
-    else
+    else if (!UseSkipList || !DO_SKIP_WINDOW_LIST(cfgpacket))
     {
       AddItem(&windows, cfgpacket->w,
 	      cfgpacket, cfgpacket->desk, Count++);
@@ -804,7 +809,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	  SendText(Fvwm_fd, buff, 0);
 	}
 	temp->needsupdate = 1;
-  	temp->iconified = iconified;
+	temp->iconified = iconified;
 	DrawButtonArray(&buttons, 0);
       }
       if (AnimCommand && AnimCommand[0] != 0)
@@ -873,7 +878,8 @@ void redraw_buttons()
 
   for (item=windows.head; item; item=item->next)
   {
-    if ((DeskNumber == item->Desk || IS_STICKY(item)) &&
+    if ((DeskNumber == item->Desk || IS_STICKY(item) ||
+	 (IS_ICONIFIED(item) && IS_ICON_STICKY(item))) &&
 	(!DO_SKIP_WINDOW_LIST(item) || !UseSkipList))
     {
       AddButton(&buttons, item->name, &(item->p), BUTTON_UP, item->count,
@@ -1052,10 +1058,10 @@ static void ParseConfigLine(char *tline)
     switch(index)
     {
     case 0: /* Font */
-      CopyString(&font_string, rest);
+      CopyStringWithQuotes(&font_string, rest);
       break;
-    case 1: /* Selfont */
-      CopyString(&selfont_string, rest);
+    case 1: /* SelFont */
+      CopyStringWithQuotes(&selfont_string, rest);
       break;
     case 2: /* Geometry */
       while (isspace((unsigned char)*rest) && *rest != '\n' && *rest != 0)
@@ -1177,16 +1183,17 @@ static void ParseConfigLine(char *tline)
 static RETSIGTYPE
 Alarm(int nonsense)
 {
-
-  switch(AlarmSet)
+  if (AlarmSet & SHOW_TIP)
   {
-  case SHOW_TIP:
     tip_window_alarm = True;
-    break;
-
-  case HIDE_TASK_BAR:
-      hide_taskbar_alarm = True;
-    break;
+  }
+  if (AlarmSet & HIDE_TASK_BAR)
+  {
+    hide_taskbar_alarm = True;
+  }
+  if (AlarmSet & STICK_TASK_BAR)
+  {
+    stick_taskbar_alarm = True;
   }
 
   AlarmSet = NOT_SET;
@@ -1207,7 +1214,12 @@ DoAlarmAction(void)
     hide_taskbar_alarm = False;
     HideTaskBar();
   }
-  else if (tip_window_alarm)
+  if (stick_taskbar_alarm)
+  {
+    stick_taskbar_alarm = False;
+    WarpTaskBar(auto_stick_y);
+  }
+  if (tip_window_alarm)
   {
     tip_window_alarm = False;
     if (AutoHide && WindowState == 0)
@@ -1258,8 +1270,8 @@ void CheckForTip(int x, int y)
     num = LocateButton(&buttons, x, y, &bx, &by, &name, &trunc);
     if (num != -1) {
       if ((Tip.type != num)  ||
-          (Tip.text == NULL) || (!trunc && (strcmp(name, Tip.text) != 0)))
-        PopupTipWindow(bx+3, by, name);
+	  (Tip.text == NULL) || (!trunc && (strcmp(name, Tip.text) != 0)))
+	PopupTipWindow(bx+3, by, name);
       Tip.type = num;
     } else {
       Tip.type = NO_TIP;
@@ -1271,7 +1283,7 @@ void CheckForTip(int x, int y)
       SetAlarm(SHOW_TIP);
   }
   else {
-    ClearAlarm();
+    ClearAlarm(SHOW_TIP);
     if (Tip.open) ShowTipWindow(0);
   }
 }
@@ -1414,7 +1426,7 @@ void LoopOnEvents(void)
 
     case LeaveNotify:
       NewTimestamp = Event.xcrossing.time;
-      ClearAlarm();
+      ClearAlarm(SHOW_TIP);
       if (Tip.open)
 	ShowTipWindow(0);
 
@@ -1488,7 +1500,8 @@ void LoopOnEvents(void)
       {
 	if (AutoStick)
 	{
-	  WarpTaskBar(Event.xconfigure.y, 0);
+	  SetAlarm(STICK_TASK_BAR);
+	  auto_stick_y = Event.xconfigure.y;
 	}
 	else
 	{
@@ -1724,7 +1737,7 @@ static void CreateOrUpdateGCs(void)
 
    /* brighting */
    gcmask = GCForeground | GCBackground | GCTile |
-            GCFillStyle  | GCGraphicsExposures;
+	    GCFillStyle  | GCGraphicsExposures;
    if (focuscolorset >= 0 || FocusBackColor != NULL)
      gcval.foreground = pfocusback;
    else
@@ -1754,7 +1767,7 @@ static Bool change_colorset(int cset, Bool force)
     {
       SetWindowBackground(
 	dpy, win, win_width, win_height, &Colorset[colorset],
-	Pdepth,	graph, True);
+	Pdepth, graph, True);
     }
     do_redraw = True;
   }
@@ -1776,8 +1789,6 @@ void StartMeUp(void)
    XSetWindowAttributes attr;
 #ifdef I18N_MB
   char **ml;
-  int mc;
-  char *ds;
   XFontStruct **fs_list;
 #endif
 
@@ -1818,27 +1829,19 @@ void StartMeUp(void)
      selfont_string = font_string;
 
 #ifdef I18N_MB
-   if ((ButtonFontset=XCreateFontSet(dpy,font_string,&ml,&mc,&ds)) == NULL) {
-#ifdef STRICTLY_FIXED
-     if ((ButtonFontset=XCreateFontSet(dpy,"fixed",&ml,&mc,&ds)) == NULL) {
-#else
-     if ((ButtonFontset=XCreateFontSet(dpy,"-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*",&ml,&mc,&ds)) == NULL) {
-#endif
-       fprintf(stderr, "%s: Couldn't load fixed font. Exiting!\n",Module);
-       exit(1);
-     }
+   if ((ButtonFontset=GetFontSetOrFixed(dpy,font_string)) == NULL)
+   {
+	   fprintf(stderr,
+		   "%s: Couldn't load font. Exiting!\n",Module);
+	   exit(1);
    }
    XFontsOfFontSet(ButtonFontset,&fs_list,&ml);
    ButtonFont = fs_list[0];
-   if ((SelButtonFontset = XCreateFontSet(dpy,selfont_string,&ml,&mc,&ds))
-       == NULL) {
-#ifdef STRICTLY_FIXED
-     if ((SelButtonFontset=XCreateFontSet(dpy,"fixed",&ml,&mc,&ds)) == NULL)
-#else
-     if ((SelButtonFontset=XCreateFontSet(dpy,"-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*",&ml,&mc,&ds)) == NULL)
-#endif
-       fprintf(stderr, "%s: Couldn't load fixed font. Exiting!\n",Module);
-       exit(1);
+   if ((SelButtonFontset=GetFontSetOrFixed(dpy,selfont_string)) == NULL)
+   {
+	   fprintf(stderr,
+		   "%s: Couldn't load selfont. Exiting!\n",Module);
+	   exit(1);
    }
    XFontsOfFontSet(SelButtonFontset,&fs_list,&ml);
    SelButtonFont = fs_list[0];
@@ -1994,8 +1997,8 @@ void StartMeUp(void)
    }
 
    XSelectInput(dpy,win,(ExposureMask | KeyPressMask | PointerMotionMask |
-                         EnterWindowMask | LeaveWindowMask |
-                         StructureNotifyMask));
+			 EnterWindowMask | LeaveWindowMask |
+			 StructureNotifyMask));
 
    InitGoodies();
    atexit(ShutMeDown);
@@ -2077,12 +2080,12 @@ PropMwmHints prop;
 /***********************************************************************
  WarpTaskBar -- Enforce AutoStick feature
  ***********************************************************************/
-void WarpTaskBar(int y, Bool force)
+static void WarpTaskBar(int y)
 {
   /* The tests on y are really useful ! */
   if (!AutoHide &&
       ((y != screen_g.y + screen_g.height - win_height - win_border &&
-	y !=  screen_g.y + win_border) || force)) {
+	y !=  screen_g.y + win_border))) {
     if (y > Midline)
     {
       win_y = screen_g.height - win_height - win_border;
@@ -2099,12 +2102,7 @@ void WarpTaskBar(int y, Bool force)
 
   if (AutoHide)
      SetAlarm(HIDE_TASK_BAR);
-
-  /* Prevent oscillations caused by race with
-     time delayed TaskBarHide().  Is there any way
-     to prevent these Xevents from being sent
-     to the server in the first place? */
-  PurgeConfigEvents();
+  ClearAlarm(STICK_TASK_BAR);
 }
 
 /***********************************************************************
@@ -2132,7 +2130,7 @@ void RevealTaskBar()
   int new_win_y;
   int inc_y = 2;
 
-  ClearAlarm();
+  ClearAlarm(HIDE_TASK_BAR);
    /* do not reveal if the taskbar is already revealed */
   if (WindowState >= 0)
     return;
@@ -2158,7 +2156,6 @@ void RevealTaskBar()
       SleepALittle();
     }
   }
-
   win_y = new_win_y;
   XMoveWindow(dpy, win, win_x, win_y);
   WindowState = 0;
@@ -2175,7 +2172,7 @@ void HideTaskBar()
   int d_x, d_y, wx, wy;
   unsigned int mask;
 
-  ClearAlarm();
+  ClearAlarm(HIDE_TASK_BAR);
   /* do not hide if the taskbar is already hiden */
   if (WindowState == -1 || win_is_shaded)
     return;
@@ -2226,10 +2223,10 @@ void HideTaskBar()
  SetAlarm -- Schedule a timeout event
  ************************************************************************/
 static void
-SetAlarm(int event)
+SetAlarm(tb_alarm_t event)
 {
   alarm(0);  /* remove a race-condition */
-  AlarmSet = event;
+  AlarmSet |= event;
   alarm(1);
 }
 
@@ -2237,22 +2234,13 @@ SetAlarm(int event)
  ClearAlarm -- Disable timeout events
  ************************************************************************/
 static void
-ClearAlarm(void)
+ClearAlarm(tb_alarm_t event)
 {
-  AlarmSet = NOT_SET;
-  alarm(0);
-}
-
-/***********************************************************************
- PurgeConfigEvents -- Wait for and purge ConfigureNotify events.
- ************************************************************************/
-void PurgeConfigEvents(void)
-{
-  XEvent Event;
-
-  XPeekEvent(dpy, &Event);
-  while (XCheckTypedWindowEvent(dpy, win, ConfigureNotify, &Event))
-    ;
+  AlarmSet &= ~event;
+  if (AlarmSet == 0)
+  {
+    alarm(0);
+  }
 }
 
 /************************************************************************
